
import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def getChannelVar(nombre:str=None, guild_id:str=None, channel_id:str=None, *args, **kwargs):
    """Obtiene una variable para un canal específico."""
    context = utils.ContextAmisynth()
    
    if nombre is None:
        raise ValueError(":x: Error, el nombre está vacío en la función `$getChannelVar[?]`.")
    
    if guild_id is None:
        guild_id = context.guild_id

    if channel_id is None:
        channel_id = context.channel_id

    var = utils.VariableManager()

    # Obtiene el valor para el canal especificado
    value = var.get_value("channel", key=nombre, guild_id=guild_id, channel_id=channel_id)
    
    if value is None:
        return f":x: No se encontró la variable `{nombre}` para el canal especificado."
    
    return value


import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def ThreadMembers(id: str = None, *args, **kwargs):
    """
    Devuelve una lista de nombres de usuario (usernames) de los miembros de un hilo (Thread).
    Si no se proporciona un ID, devuelve los miembros desde el contexto.
    """
    context = utils.ContextAmisynth()
    obj_bot = utils.bot_inst

    # Si no se proporciona un ID, obtener los miembros del contexto
    if id is None:
        members = context.members_name
    else:
        obj_thread = await obj_bot.fetch_channel(id)


        fetched_members = await obj_thread.fetch_members()
        members = fetched_members.members if hasattr(fetched_members, "members") else fetched_members
    # Convertir a lista de nombres de usuario
    usernames = [member.name for member in members]
    return usernames



import xfox
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def userAvatar(id=None, *args, **kwargs):
    context = utils.ContextAmisynth()

    if id is None:
        n = int(context.author_id)

    else:
        n = int(id)
        
        
    avatar = context.get_user_avatar_by_id(n)

    return avatar
    




import xfox
import re

@xfox.addfunc(xfox.funcs)
async def findURLs(text: str, *args, **kwargs):
    # Expresión regular para detectar URLs
    url_pattern = r"https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+"
    urls = re.findall(url_pattern, text)
    
    return ', '.join(urls) if urls else ""



import xfox
import discord
from Amisynth.utils import buttons, menu_options
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def newSelectMenu(menu_id: str, min_val: int, max_val: int, placeholder: str, message_id=None, *args, **kwargs):
    
    context = utils.ContextAmisynth()
    # Obtener las opciones del menú por su ID desde menu_options
    options = menu_options.get(menu_id, [])

    # Crear el menú de selección
    select_menu = discord.ui.Select(
        placeholder=placeholder if placeholder else "Seleccione una opción",
        min_values=min_val,
        max_values=max_val,
        options=options,
        custom_id=menu_id
    )

    if message_id is None:
        buttons.append(select_menu)  # Si no hay mensaje, guardar el botón en la lista
    else:

        message = await context.get_message_from_id(int(message_id))


        # Recuperar o crear una vista nueva
        view = discord.ui.View() if not message.components else discord.ui.View.from_message(message)
        view.add_item(select_menu)  # Agregar el botón a la vista

        await message.edit(view=view)  # Editar el mensaje con la nueva vista
    
    return ""



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def footer(texto_footer: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda un footer en la lista de embeds, con un texto de footer específico y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param texto_footer: El texto que se quiere mostrar en el footer del embed.
    :param indice: El índice opcional del embed (posición en la lista).
    """
    if texto_footer is None:
        raise ValueError(":x: Error argumento primero de `$footer[?;..]` vacio")
    # Crear el embed con el footer
    embed = {
        "footer": texto_footer,
        "index": indice  # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el footer
    for i, item in enumerate(embeds):
        if item["index"] == indice:
            # Mantener los otros atributos del embed y solo actualizar el footer
            embeds[i]["footer"] = texto_footer
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def nickname(id: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    if id is None:
        username  = context.display_name
    else:
        username = context.get_nickname_by_id(int(id))

    return username



import discord
from Amisynth.utils import menu_options
# Diccionario para almacenar las opciones de cada menú por su ID
import xfox

@xfox.addfunc(xfox.funcs)
async def addSelectMenuOption(menu_id: str, label: str, value: str, description: str, *args, **kwargs):
    """Agrega una opción al menú de selección, almacenándola en una lista por ID."""
    option = discord.SelectOption(label=label, value=value, description=description)
    
    # Si no existe una lista de opciones para este ID, la crea
    if menu_id not in menu_options:
        menu_options[menu_id] = []
    
    # Agregar la opción al menú correspondiente
    menu_options[menu_id].append(option)
    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def getUserVar(nombre:str=None, user_id:str=None, guild_id:str=None,  *args, **kwargs):
    context = utils.ContextAmisynth()
    if nombre is None:
        raise ValueError(":x: Error el nombre esta vacio de la `$setUserVar[?;..]`")
    
    if user_id is None:
        user_id = context.author_id

    if guild_id is None:
        guild_id = context.guild_id

    var = utils.VariableManager()

    value = var.get_value(level="user", key=str(nombre), guild_id=str(guild_id), user_id=str(user_id))
    return value


import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global

@xfox.addfunc(xfox.funcs)
async def textSplit(text: str, separator: str = None, *args, **kwargs):
    global split_storage

    if separator is None or separator == "":  
        split_storage["last_split"] = list(text)  # Guardar cada letra en la lista
    else:
        split_storage["last_split"] = text.split(separator)  # Separar por el delimitador

    return ""



import xfox

@xfox.addfunc(xfox.funcs)
async def getMessageDelete(*args, **kwargs):

    if "ctx_message_delete_env" in kwargs:
        message = kwargs["ctx_message_delete_env"]
        return message.content
    return ""


import discord
import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def sendMessage(texto, retornar_id="false", canal_id=None, *args, **kwargs): 
    n = utils.ContextAmisynth()
    # Si no se pasa un canal_id, intenta obtenerlo desde el contexto
    if canal_id is None:
        canal = await n.get_channel(int(n.channel_id))  # Obtener el canal desde el contexto
       
    else:
        canal = await n.get_channel(int(canal_id))  # Obtener el canal desde el ID pasado
    
    if canal is None:
        raise ValueError(":x: Error en obtener el canal ID, Contacte con Soporte.")

    # Verificar si el canal es válido antes de enviar el mensaje
    if isinstance(canal, discord.TextChannel):
        mensaje = await canal.send(texto)
        if str(retornar_id).lower() == "true":
            return mensaje.id  # Retorna el ID del mensaje si se solicita
        
        return ""  # Retorna una cadena vacía si no se necesita el ID

    print(f"No se encontró un canal válido para enviar el mensaje. Canal ID: {canal_id}")
    return None  # Indicar que falló el envío



import xfox

@xfox.addfunc(xfox.funcs, name="porcent")
async def percentage_of_number(number: str, percent: str, **kwargs):
    try:
        num = float(number)
        percent_value = float(percent)
        return str((num * percent_value) / 100)
    except ValueError:
        return ":x: Invalid number value en `$porcent[]`"



import xfox

@xfox.addfunc(xfox.funcs)
async def editChannelBefore(option=None, *args, **kwargs):
    if "ctx_guild_channel_edit" in kwargs:
        after = kwargs["ctx_guild_channel_edit"][0]
        return getattr(after, option, "")
    return ""



import xfox
import discord
import Amisynth.utils as utils


@xfox.addfunc(xfox.funcs, name="createChannel")
async def create_channel(name: str, type: str, category_id: int = None, retornar:bool=False, *args, **kwargs):
    contexto = utils.ContextAmisynth()
    guild = contexto.obj_guild  # Obtiene el servidor (guild)

    # Validar el tipo de canal
    valid_types = ["category", "text", "voice", "stage", "forum"]
    if type not in valid_types:
        return f"❌ Tipo de canal inválido. Usa uno de los siguientes: {', '.join(valid_types)}."

    # Buscar la categoría si se proporcionó un ID
    category = None
    if category_id:
        category = discord.utils.get(guild.categories, id=category_id)
        if not category:
            return f"❌ No se encontró la categoría con ID {category_id}."

    try:
        # Crear el canal según el tipo proporcionado
        if type == "text":
            channel = await guild.create_text_channel(name, category=category)
        elif type == "voice":
            channel = await guild.create_voice_channel(name, category=category)
        elif type == "category":
            channel = await guild.create_category_channel(name)
        elif type == "stage":
            channel = await guild.create_stage_channel(name, category=category)
        elif type == "forum":
            channel = await guild.create_forum_channel(name, category=category)

        if retornar is True or retornar is "true" or retornar is "yes":
            return channel.id
        else:
            return ""

    except discord.Forbidden:
        return "❌ No tengo permisos suficientes para crear un canal."

    except discord.HTTPException as e:
        return f"❌ Ocurrió un error al crear el canal: {e}"

    except Exception as e:
        return f"❌ Error inesperado: {e}"



import xfox
from Amisynth.utils import json_storage
import asyncio
@xfox.addfunc(xfox.funcs)
async def json(*args, **kwargs):
    try:
        data = json_storage  # Inicia con el JSON base

        for clave in args:
            if isinstance(data, dict) and clave in data:
                data = data[clave]  # Acceder a clave en diccionario
            elif isinstance(data, list):
                try:
                    index = int(clave)  # Convertir clave a entero si es índice
                    data = data[index]  # Acceder a índice en lista
                except ValueError:
                    return f""
                    
                except IndexError:
                    return f""
            else:
                return f""
        
        return data  # Devuelve el resultado final
    except Exception as e:
        raise ValueError(f"Error inesperado: {str(e)}")




import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def ThreadID(name: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()  # ← si es async
    obj_bot = utils.bot_inst

    # Si no se proporciona un nombre, usamos el hilo actual
    if name is None:
        return context.thread_id  # Asumiendo que context.thread_name es el nombre del hilo actual
    else:
        # Buscar entre los hilos activos del canal
        channel = context.channel
        threads = await channel.active_threads()
        for thread in threads:
            if thread.name == name:
                return thread.id
        return None  # No se encontró hilo con ese nombre



import xfox
import discord
from Amisynth.utils import buttons
import Amisynth.utils as utils

# Contador de fila global
row_counter = 0  

@xfox.addfunc(xfox.funcs)
async def addButton(new_row: str, button_id: str, label: str, style: str, disabled="false", emoji=None, message_id=None, *args, **kwargs):
    """Crea múltiples botones interactivos y devuelve una lista de objetos de botones creados."""
    context = utils.ContextAmisynth()
    
    global row_counter  # Para modificar el contador de fila

    # Estilos disponibles
    estilos = {
        "primary": discord.ButtonStyle.primary,
        "secondary": discord.ButtonStyle.secondary,
        "success": discord.ButtonStyle.success,
        "danger": discord.ButtonStyle.danger,
        "link": discord.ButtonStyle.link
    }
    
    button_style = estilos.get(style, discord.ButtonStyle.primary)

    # Validar el parámetro 'disabled'
    if disabled in ["true", "false"]:
        disabled = disabled == "true"
    else:
        raise ValueError("Error en el parámetro $addButton[]")

    # Validar si es un botón de tipo enlace
    custom_id = button_id if button_style != discord.ButtonStyle.link else None
    url = button_id if button_style == discord.ButtonStyle.link else None

    # Manejo de la fila (row)
    if new_row.lower() == "true":
        row_counter += 1
    elif new_row.lower() == "re":
        row_counter = 0  

    button = discord.ui.Button(
        label=label,
        custom_id=custom_id,
        style=button_style,
        emoji=emoji,
        disabled=disabled,
        url=url,
        row=row_counter
    )

    if message_id is None:
        buttons.append(button)  # Si no hay mensaje, guardar el botón en la lista
    else:

        message = await context.get_message_from_id(int(message_id))


        # Recuperar o crear una vista nueva
        view = discord.ui.View() if not message.components else discord.ui.View.from_message(message)
        view.add_item(button)  # Agregar el botón a la vista

        await message.edit(view=view)  # Editar el mensaje con la nueva vista

    return ""



import discord
import re
from discord import app_commands
from discord.ext import commands
import json

# Lista global de embeds
embeds = []  # Esta lista se puede actualizar desde otros archivos, como 'image.py' o 'thumbnail.py'

buttons = []  # Si también necesitas manejar botones, puedes añadir la lógica aquí

modales = []

modals_activate = False

ephemeral = False

options_slash = []


json_storage = {}


menu_options = {}

message_id = None

async def set_message_bot(msg):
    message_id = msg

bot_inst = None


# utils.py

# Variable global para almacenar las cabeceras y los datos JSON
http_data = {
    "headers": {},
    "json": {}
}

# Variable para almacenar la respuesta HTTP
http_response = ""

http_status = None

async def utils():
    nuevos_embeds = []  # Usamos una lista nueva para los embeds procesados

    # Iterar sobre cada item en la lista de embeds

    for item in embeds:
        embed = discord.Embed()  # Crear el embed sin título y descripción por defecto

        if "color" in item and item["color"]:
            embed.color = item["color"]

        # Verificar si el título está presente y agregarlo
        if 'title' in item and item['title']:
            embed.title = item['title']

        if "title_url" in item and item["title_url"]:
            embed.url = ensure_double_slash(item["title_url"] or "")

        # Verificar si la descripción está presente y agregarla
        if 'description' in item and item['description']:
            embed.description = item['description']

        # Verificar si hay imagen y añadirla, asegurando que la URL esté bien formada
        if "image" in item and item["image"]:
            n = ensure_double_slash(item["image"])
            embed.set_image(url=n)

        if "thumbnail_icon" in item and item["thumbnail_icon"]:
            n = ensure_double_slash(item["thumbnail_icon"])
            embed.set_thumbnail(url=n)

        if "footer" in item and item["footer"]:
            icon = item["footer_icon"] if "footer_icon" in item and item["footer_icon"] else None
            embed.set_footer(text=item["footer"], icon_url=icon)

        if "timestamp" in item and item["timestamp"]:
            from datetime import datetime
            embed.timestamp = datetime.utcnow()
        

        author_name = item.get("author")
        author_icon = ensure_double_slash(item.get("author_icon", ""))
        author_url = ensure_double_slash(item.get("author_url", ""))

        if author_name or author_icon or author_url:
            embed.set_author(name=author_name or "", icon_url=author_icon, url=author_url)
        # Agregar el embed procesado a la lista

        if "fields" in item and isinstance(item["fields"], list):
            for field in item["fields"]:
                embed.add_field(
                    name=field.get("name"),
                    value=field.get("value"),
                    inline=field.get("inline"))


        nuevos_embeds.append(embed)

    # Retornar los nuevos embeds procesados y los botones
    return buttons, nuevos_embeds



def ensure_double_slash(text: str) -> str:
    return re.sub(r"https:(?!//)", "https://", text)



from discord import Interaction, Message, Member, TextChannel, RawReactionActionEvent
from discord.ext import commands
from discord.utils import get


class ContextAmisynth:
    _instances = {}  
    _last_ctx = None  

    def __new__(cls, ctx=None):
        
        if ctx:
            cls._last_ctx = ctx  
            ctx_type = type(ctx).__name__
            if ctx_type not in cls._instances:
                
                cls._instances[ctx_type] = super().__new__(cls)
            return cls._instances[ctx_type]
        
        return cls._instances.get(type(cls._last_ctx).__name__, super().__new__(cls))  

    def __init__(self, ctx=None):
        
        if ctx:
            self.set_context(ctx)

    def set_context(self, ctx):
        """Establece o cambia el contexto actual."""
        
        self.ctx = ctx
        ContextAmisynth._last_ctx = ctx 
        ContextAmisynth._instances[type(ctx).__name__] = self 


        if isinstance(ctx, commands.Context):
            self.author_id = ctx.author.id
            self.guild_id = ctx.guild.id if ctx.guild else None
            self.channel_id = ctx.channel.id
            self.message_id = ctx.message.id
            self.username = ctx.author.name
            self.arguments = ctx.message.content.split()[1:]
            self.user_roles = [role.id for role in ctx.author.roles if role.name != "@everyone"]
            self.message_content = ctx.message.content[1:]
            self.display_name = ctx.author.display_name
            self.channel_name= ctx.channel.name
            self.member_count = len(ctx.channel.members)
            self.obj_channel = ctx.channel
            self.obj_guild = ctx.guild
            self.obj_message = ctx.message
 
        elif isinstance(ctx, discord.Interaction):
            self.author_id = ctx.user.id
            self.guild_id = ctx.guild.id if ctx.guild else None
            self.channel_id = ctx.channel.id if ctx.channel else None
            self.message_id = ctx.message.id if ctx.message else None
            self.username = ctx.user.name
            self.custom_id = ctx.data.get("custom_id")
            self.slash_options = ctx.data.get("options")
            self.user_roles = [role.id for role in ctx.user.roles if role.name != "@everyone"]
            self.display_name = ctx.user.display_name
            
            self.obj_channel = ctx.channel
            self.obj_member = ctx.user

        elif isinstance(ctx, Message):
            self.author_id = ctx.author.id
            self.guild_id = ctx.guild.id if ctx.guild else None
            self.channel_id = ctx.channel.id
            self.message_id = ctx.id
            self.username = ctx.author.name
            self.arguments = ctx.content.split()
            self.user_roles = [role.id for role in ctx.author.roles if role.name != "@everyone"]
            self.message_content = ctx.content
            self.display_name = ctx.author.display_name
            self.obj_channel = ctx.channel
            self.obj_channel = ctx.author

        elif isinstance(ctx, Member):
            self.author_id = ctx.id  # ID del usuario
            self.guild_id = ctx.guild.id if ctx.guild else None  # ID del servidor (si está en uno)
            self.username = ctx.name  # Nombre de usuario
            self.display_name = ctx.display_name  # Nombre mostrado en el servidor
            self.obj_member = ctx

        elif isinstance(ctx, discord.TextChannel):
            self.channel_id = ctx.id
            self.guild_id = ctx.guild.id if ctx.guild else None
            self.channel_name = ctx.name
            self.obj_channel = ctx

            
        elif isinstance(ctx, RawReactionActionEvent):
            self.author_id = ctx.user_id  
            self.guild_id = ctx.guild_id  
            self.channel_id = ctx.channel_id  
            self.message_id = ctx.message_id




            self.emoji_name = ctx.emoji.name  # Nombre del emoji
            self.emoji_id = ctx.emoji.id  # ID del emoji, si es un emoji personalizado
            self.is_custom = ctx.emoji.is_custom_emoji()  # Verificar si es un emoji personalizado
            self.user_roles = [role.id for role in ctx.member.roles if role.name != "@everyone"]

            


        elif isinstance(ctx, discord.Thread):
   
            self.obj_guild = ctx.guild
            self.channel_id = ctx.parent.id
            self.obj_channel = ctx.parent

            self.channel_name = ctx.name

            self.thread_id = ctx.id
            self.thread_name = ctx.name
            self.thread_owner = ctx.owner.id if ctx.owner else ctx.owner_id  # fallback
            self.guild_id = ctx.guild.id
  
            self.archived = ctx.archived
            self.locked = ctx.locked
            self.invitable = ctx.invitable  # Solo relevante en hilos privados
            self.auto_archive_duration = ctx.auto_archive_duration
            self.created_at = ctx.created_at
            self.archived_at = ctx.archive_timestamp
            self.slowmode_delay = ctx.slowmode_delay
            self.message_count = ctx.message_count
            self.member_count = ctx.member_count
            self.last_message_id = ctx.last_message_id

            self.members_name = ctx.members
            self.thread_type = ctx.type.name

   
            self.bot_member_info = ctx.me  #
    
   
            self.parent_name = ctx.parent.name


        else:
            self.author_id = None
            self.guild_id = None
            self.channel_id = None
            self.message_id = None
    
    def get_user_id_by_username(self, username: str):
        """Obtiene el ID de un usuario a partir de su nombre de usuario dentro del contexto."""
        if not self.guild_id:
            return None  # No hay un servidor asociado

        guild = self.ctx.guild  # Obtener la guild desde el contexto

        if guild:
            member = get(guild.members, name=username)  # Buscar por nombre de usuario
            if member:
                return member.id

        return None  # No se encontró el usuario
    



    def get_channel_id_by_name(self, channel_name: str):
        """Obtiene el ID de un canal a partir de su nombre dentro del contexto."""
        if not self.guild_id:
            return None  # No hay un servidor asociado

        guild = self.ctx.guild  # Obtener la guild desde el contexto

        if guild:
            channel = get(guild.channels, name=channel_name)  # Buscar por nombre del canal
            if channel:
                return channel.id  # Retorna el ID del canal encontrado

        return None  # No se encontró el canal



    def get_username_by_id(self, user_id: int):
        """Obtiene el nombre de usuario a partir de un ID dentro del contexto del servidor."""
        if not self.guild_id:
            return None  # No hay un servidor asociado

        guild = self.ctx.guild  # Obtener la guild desde el contexto

        if guild:
            member = get(guild.members, id=user_id)  # Buscar por ID de usuario
            if member:
                return member.name  # Retorna el nombre de usuario
    
        return None  # Usuario no encontrado
    
    def get_nickname_by_id(self, user_id: int):
        """Obtiene el nombre de usuario a partir de un ID dentro del contexto del servidor."""
        if not self.guild_id:
            return None  # No hay un servidor asociado

        guild = self.ctx.guild  # Obtener la guild desde el contexto

        if guild:
            member = get(guild.members, id=user_id)  # Buscar por ID de usuario
            if member:
                return member.display_name  # Retorna el nombre de usuario
    
        return None  # Usuario no encontrado
    

    
    def get_mentions(self):
        """Devuelve una lista de IDs de usuarios mencionados en el mensaje actual del contexto."""
        if isinstance(self.ctx, commands.Context):
            return [user.id for user in self.ctx.message.mentions]  # Extrae IDs de menciones

        elif isinstance(self.ctx, Interaction) and self.ctx.message:
            return [user.id for user in self.ctx.message.mentions]  # Si Interaction tiene un mensaje

        elif isinstance(self.ctx, Message):
            return [user.id for user in self.ctx.mentions]  # Extrae menciones en mensajes directos

        return []
    
    def get_role_mentions(self):
        """Devuelve una lista de IDs de roles mencionados en el mensaje actual del contexto."""
        if isinstance(self.ctx, commands.Context):
            return [role.id for role in self.ctx.message.role_mentions]  # Extrae IDs de roles mencionados

        elif isinstance(self.ctx, Interaction) and self.ctx.message:
            return [role.id for role in self.ctx.message.role_mentions]  # Si Interaction tiene un mensaje

        elif isinstance(self.ctx, Message):
            return [role.id for role in self.ctx.role_mentions]  # Extrae menciones de roles en mensajes directos

        return []
    
    def get_channel_mentions(self):
        """Devuelve una lista de IDs de canales mencionados en el mensaje actual del contexto."""
        if isinstance(self.ctx, commands.Context):
            return [channel.id for channel in self.ctx.message.channel_mentions]  # Extrae IDs de canales mencionados

        elif isinstance(self.ctx, Interaction) and self.ctx.message:
            return [channel.id for channel in self.ctx.message.channel_mentions]  # Si Interaction tiene un mensaje

        elif isinstance(self.ctx, Message):
            return [channel.id for channel in self.ctx.channel_mentions]  # Extrae menciones de canales en mensajes directos

        return []
    

    def get_text_channels_name(self, guild_id: int):
        """Devuelve una lista de nombres de los canales de texto de un servidor, excluyendo las categorías."""
        guild = self.ctx.guild  # Obtiene el servidor desde el contexto
        if not guild or guild.id != guild_id:
             return []  # Si el servidor no coincide, devuelve una lista vacía

        return [channel.name for channel in guild.channels if not isinstance(channel, discord.CategoryChannel)]
    


    def get_text_channels_ids(self, guild_id: int):
        """Devuelve una lista de nombres de los canales de un servidor, excluyendo las categorías."""
        guild = self.ctx.guild  # Obtiene el servidor desde el contexto
        if not guild or guild.id != guild_id:
            return []  # Si el servidor no coincide, devuelve una lista vacía

        return [str(channel.id) for channel in guild.channels if not isinstance(channel, discord.CategoryChannel)]
    

    def get_categorys_names(self, guild_id: int):
        """Devuelve una lista de nombres de las categorías de un servidor."""
        guild = self.ctx.guild  # Obtiene el servidor desde el contexto
        if not guild or guild.id != guild_id:
            return []  # Si el servidor no coincide, devuelve una lista vacía

        return [category.name for category in guild.channels if isinstance(category, discord.CategoryChannel)]

    def get_categorys_ids(self, guild_id: int):
        """Devuelve una lista de IDs de las categorías de un servidor."""
        guild = self.ctx.guild  # Obtiene el servidor desde el contexto
        if not guild or guild.id != guild_id:
            return []  # Si el servidor no coincide, devuelve una lista vacía

        return [str(category.id) for category in guild.channels if isinstance(category, discord.CategoryChannel)]


    
    async def get_message_from_id(self, message_id: int):
        """Busca un mensaje en cualquier canal del servidor actual."""
    
        if not hasattr(self, "ctx") or not self.ctx.guild:
            return None  # Asegurarse de que haya contexto y sea un servidor válido
    
        for channel in self.ctx.guild.text_channels:  # Iterar sobre los canales de texto
            try:
                message = await channel.fetch_message(message_id)  # Intentar obtener el mensaje
                return message  # Si se encuentra, retornarlo
            except discord.NotFound:
                continue  # Si no se encuentra, seguir con el siguiente canal
            except discord.Forbidden:
                print(f"No tengo permisos para leer {channel.name}")  # Diagnóstico de permisos
                continue
            except discord.HTTPException as e:
                print(f"Error HTTP en {channel.name}: {e}")
                continue

        return None  # Si no se encuentra en ningún canal, devolver None


    
    async def get_channel(self, channel_id:int=None):
        """Obtiene el objeto del canal dado un ID o el canal almacenado en el contexto."""
        if not hasattr(self, "ctx"):
            return None  # No hay contexto disponible

        client = self.ctx._state._get_client()  # Obtiene el cliente (bot)
        channel_id = channel_id or getattr(self, "channel_id", None)  # Usa el ID dado o el almacenado en el contexto

        if channel_id:
            return client.get_channel(channel_id)  # Retorna el objeto del canal si está en caché

        return None  # Retorna None si no se encuentra el canal
    
    async def modificar_rol(self, user_id: int, role_id: int, action: str):
        """
        Modifica el rol de un usuario en el servidor.

        :param user_id: ID del usuario.
        :param role_id: ID del rol.
        :param action: 'agregar' para añadir el rol, 'remover' para quitarlo.
        """
        if not self.ctx:
            return
        
        guild = self.ctx.guild  # Obtiene el servidor actual
        if not guild:
            return
        
        member = guild.get_member(user_id)
        if not member:
            await self.ctx.send("❌ Usuario no encontrado.")
            return
        
        role = guild.get_role(role_id)
        if not role:
            await self.ctx.send("❌ Rol no encontrado.")
            return

        if action == "+":
            await member.add_roles(role)
        elif action == "-":
            await member.remove_roles(role)

    def get_user_avatar_by_id(self, user_id: int):
        """Obtiene el avatar de un usuario a partir de su ID dentro del contexto."""
        if not self.guild_id:
            return None  # No hay un servidor asociado

        guild = self.ctx.guild  # Obtener la guild desde el contexto

        if guild:
            member = guild.get_member(user_id)  # Buscar por ID de usuario
            if member:
                return member.avatar.url if member.avatar else member.default_avatar.url

        return None  # No se encontró el usuario o no tiene avatar personalizado




import os
import json  # Asegúrate de importar el módulo json

class VariableManager:
    _instances = {}  # Diccionario para manejar instancias por archivo JSON

    def __new__(cls, path="data.json"):
        if path not in cls._instances:
            cls._instances[path] = super(VariableManager, cls).__new__(cls)
            cls._instances[path]._initialize(path)
        return cls._instances[path]

    def _initialize(self, path):
        """Carga datos desde el archivo JSON si existe, de lo contrario, inicializa una estructura vacía."""
        self._file_path = path  # Guardar la ruta del archivo JSON
        if os.path.exists(self._file_path):
            with open(self._file_path, "r", encoding="utf-8") as file:
                self.data = json.load(file)
        else:
            self.data = {
                "guilds": {},
                "channels": {},
                "users": {},
                "global": {},
                "global_users": {}
            }

    def _save_to_file(self):
        """Guarda los datos en el archivo JSON."""
        with open(self._file_path, "w", encoding="utf-8") as file:
            json.dump(self.data, file, indent=4, ensure_ascii=False)

    def set_value(self, level, key, value, guild_id=None, user_id=None, channel_id=None):
        """Establece un valor en el nivel indicado y lo guarda en el JSON."""

        if level == "guild":
            if guild_id is None:
                raise ValueError("guild_id es requerido para valores de servidor.")
            # Asegurar que no se sobrescribe, solo se añade o actualiza
            self.data["guilds"].setdefault(guild_id, {})[key] = value

        elif level == "channel":
            if guild_id is None or channel_id is None:
                raise ValueError("guild_id y channel_id son requeridos para valores de canal.")
            self.data["channels"].setdefault(guild_id, {}).setdefault(channel_id, {})[key] = value

        elif level == "user":
            if guild_id is None or user_id is None:
                raise ValueError("guild_id y user_id son requeridos para valores de usuario en un servidor.")
        # Aquí es clave: asegurar niveles sin sobrescribir
            self.data["users"].setdefault(guild_id, {}).setdefault(user_id, {})[key] = value

        elif level == "global":
            self.data["global"][key] = value

        elif level == "global_user":
            if user_id is None:
                raise ValueError("user_id es requerido para valores globales de usuario.")
            self.data["global_users"].setdefault(user_id, {})[key] = value

        else:
            raise ValueError("Nivel no válido. Usa: 'guild', 'channel', 'user', 'global', 'global_user'.")

        self._save_to_file()


    def get_value(self, level, key, guild_id=None, user_id=None, channel_id=None):
        """Obtiene un valor almacenado en el nivel indicado."""
        if level == "guild":
            return self.data["guilds"].get(guild_id, {}).get(key, None)
        
        elif level == "channel":
            return self.data["channels"].get(guild_id, {}).get(channel_id, {}).get(key, None)
        
        elif level == "user":
            return self.data["users"].get(guild_id, {}).get(user_id, {}).get(key, None)
        
        elif level == "global":
            return self.data["global"].get(key, None)
        
        elif level == "global_user":
            return self.data["global_users"].get(user_id, {}).get(key, None)

        else:
            raise ValueError("Nivel no válido.")

    def to_json(self):
        """Devuelve la estructura en formato JSON."""
        return json.dumps(self.data, indent=4, ensure_ascii=False)






async def clear_data():
        # Limpiar las listas de botones, embeds y menús
    embeds.clear()
    buttons.clear()
    menu_options.clear()
    options_slash.clear()
    modales.clear()
    modals_activate = False
    ephemeral = False



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def ThreadName(id: int = None, *args, **kwargs):
    context = utils.ContextAmisynth()  # ← si es una clase asíncrona (ajústalo si no lo es)

    obj_bot = utils.bot_inst

    if id is None:
        try:
            name = context.thread_name
        except AttributeError:
            return None
    else:
        thread = await obj_bot.fetch_channel(id)  # ← faltaba el await
        name = thread.name

    return name



import xfox
import discord
import Amisynth.utils as utils


@xfox.addfunc(xfox.funcs)
async def editButton(button_id: str, label: str, style: str, disabled="false", emoji=None, message_id=None, *args, **kwargs):
    """Edita o añade un botón en un mensaje ya enviado."""



    context = utils.ContextAmisynth()

    if message_id is None:
        message_id = int(context.message_id)
    
    estilos = {
        "primary": discord.ButtonStyle.primary,
        "secondary": discord.ButtonStyle.secondary,
        "success": discord.ButtonStyle.success,
        "danger": discord.ButtonStyle.danger,
        "link": discord.ButtonStyle.link
    }

    button_style = estilos.get(style, discord.ButtonStyle.primary)

    if disabled in ["true", "false"]:
        disabled = disabled == "true"
    else:
        raise ValueError("Parámetro 'disabled' no válido en $editButton[]")

    custom_id = button_id if button_style != discord.ButtonStyle.link else None
    url = button_id if button_style == discord.ButtonStyle.link else None

    message = await context.get_message_from_id(int(message_id))

    # Obtener vista actual
    view = discord.ui.View() if not message.components else discord.ui.View.from_message(message)

    # Buscar si el botón ya existe (por custom_id o URL)
    for item in view.children:
        if isinstance(item, discord.ui.Button):
            if (item.custom_id == custom_id and custom_id is not None) or (item.url == url and url is not None):
                # Editar propiedades del botón existente
                item.label = label
                item.style = button_style
                item.disabled = disabled
                item.emoji = emoji
                await message.edit(view=view)
                return ""

    # Si no se encontró el botón, agregar uno nuevo
    new_button = discord.ui.Button(
        label=label,
        custom_id=custom_id,
        style=button_style,
        disabled=disabled,
        emoji=emoji,
        url=url
    )
    view.add_item(new_button)

    await message.edit(view=view)
    return ""



import xfox
from Amisynth.utils import ContextAmisynth

@xfox.addfunc(xfox.funcs)
async def isBanned(user_id=None, *args, **kwargs):
    context = ContextAmisynth()
    guild = context.obj_guild
    if user_id is None or guild is None:
        return "false"
    
    try:
        bans = await guild.bans()
        return "true" if any(str(ban.user.id) == str(user_id) for ban in bans) else "false"
    except:
        return "false"



import xfox
import ast
import operator

@xfox.addfunc(xfox.funcs, name="calculate")
async def calculate_funcs(expression: str, *args, **kwargs):
    # Mapeo de operadores permitidos
    allowed_operators = {
        ast.Add: operator.add,       # +
        ast.Sub: operator.sub,       # -
        ast.Mult: operator.mul,      # *
        ast.Div: operator.truediv,   # /
        ast.Pow: operator.pow,       # **
        ast.Mod: operator.mod,       # %
        ast.USub: operator.neg,      # -
    }

    def eval_node(node):
        if isinstance(node, ast.Num):  # números
            return node.n
        elif isinstance(node, ast.BinOp):  # operaciones binarias
            left = eval_node(node.left)
            right = eval_node(node.right)
            op = allowed_operators.get(type(node.op))
            if op is None:
                raise ValueError(f"Operador no permitido en la expresión: '{ast.dump(node.op)}' en `$calculate[]`")
            return op(left, right)
        elif isinstance(node, ast.UnaryOp):  # operaciones unarias, como -5
            operand = eval_node(node.operand)
            op = allowed_operators.get(type(node.op))
            if op is None:
                raise ValueError(f"Operador unario no permitido: '{ast.dump(node.op)}'. Solo se permite '-' para números en `$calculate[]`.")
            return op(operand)
        else:
            raise ValueError(f"Expresión inválida: '{ast.dump(node)}'. Asegúrate de usar solo números y operadores permitidos en `$calculate[]`")

    try:
        # Analiza la expresión
        tree = ast.parse(expression, mode="eval")
        result = eval_node(tree.body)
        return str(result)
    except Exception as e:
        return f"Error en la expresión: {str(e)} en `$calculate[]`"



import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs, name="roleGrant")
async def role_grant(user_id: int, *roles: str, **kwargs):
    ctx = utils.ContextAmisynth()
    
    for role_data in roles:
        action = role_data[0]  # '+' o '-'
        role_id = int(role_data[1:])  # ID del rol sin el prefijo
        
        if action == '+':
            await ctx.modificar_rol(user_id, role_id, "+")
        elif action == '-':
            await ctx.modificar_rol(user_id, role_id, "-")
    
    return ""



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando
from datetime import datetime
@xfox.addfunc(xfox.funcs)
async def addTimestamp(indice=None, *args, **kwargs):
    """
    Agrega un timestamp en la lista de embeds con un índice opcional.
    Si no se especifica el índice, se agrega en la posición 1.
    
    :param args: Argumentos opcionales, el primer argumento será el índice.
    """
    if indice is None:
        indice = 1
    # Verificar si hay un argumento de índice, si no, asignar el valor predeterminado de 1
    
    embed = {
        "timestamp": "true",  # Aquí puedes definir lo que quieras almacenar en el embed
        "index": int(indice)           # El índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizarlo
    for i, item in enumerate(embeds):
        if item.get("index") == int(indice):
            # Actualizamos el embed solo si el índice coincide
            embeds[i]["timestamp"] = "true"  # Aquí puedes poner lo que quieras actualizar
            break
    else:
        # Si no se encontró, agregamos un nuevo embed con ese índice
        embeds.append(embed)

    return ""



import xfox
from Amisynth.utils import ContextAmisynth
@xfox.addfunc(xfox.funcs)
async def isAdmin(user_id=None, *args, **kwargs):
    context = ContextAmisynth()
    guild = context.obj_guild
    if user_id is None or guild is None:
        return "false"
    
    member = await guild.fetch_member(int(user_id))
    return "true" if member.guild_permissions.administrator else "false"



import aiohttp
import xfox
import Amisynth.utils as utils

# Función para agregar headers a las solicitudes HTTP
@xfox.addfunc(xfox.funcs)
async def httpAddHeader(key: str, value: str, *args, **kwargs):
    """Agrega un header a la solicitud HTTP para las solicitudes POST/PUT/PATCH."""
    if "headers" not in utils.http_data:
        utils.http_data["headers"] = {}

    # Añadir el header al diccionario de headers
    utils.http_data["headers"][key] = value
    print(F"[DEBUG HTTPADDHEADER] Header '{key}: {value}' añadido a la solicitud.")
    return f""



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonExists(*claves, **kwargs):
    if not claves:
        raise ValueError("Error: Debes proporcionar al menos una clave en $jsonExists[]")
    
    data = json_storage
    
    try:
        for clave in claves:
            if clave not in data:
                return "false"
            data = data[clave]  # Avanza en la estructura del JSON
        
        return "true"
    except (KeyError, TypeError):
        return "false"



import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global

@xfox.addfunc(xfox.funcs)
async def editSplitText(index: str, value: str, *args, **kwargs):
    global split_storage
    if "last_split" not in split_storage:
        raise ValueError("No text has been split yet")
    
    try:
        index = int(index) - 1  # Ajustar para que empiece desde 1
        if 0 <= index < len(split_storage["last_split"]):
            split_storage["last_split"][index] = value
        else:
            raise ValueError("Index out of range")
    except ValueError:
        raise ValueError("Invalid index: must be an integer")
    
    return ""



import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def findUser(query: str, fallback_to_author: bool = False, *args, **kwargs):
    context = utils.ContextAmisynth()
    guild = context.obj_guild

    if guild is None:
        print("[DEBUG FINDUSER] No se puede buscar fuera de un servidor.")
        raise ValueError(":x: No se puede buscar fuera de un servidor.")

    # Buscar por ID directo
    if query.isdigit():
        member = guild.get_member(int(query))
        if member:
            print(f"[DEBUG FINDUSER] Encontrado por ID: {member}")
            return str(member.id)

    query_lower = query.lower()
    for member in guild.members:
        if (
            member.name.lower() == query_lower or
            member.display_name.lower() == query_lower
        ):
            print(f"[DEBUG FINDUSER] Encontrado por nombre/display_name: {member}")
            return str(member.id)

    if fallback_to_author:
        print("[DEBUG FINDUSER] No se encontró, devolviendo author.id por fallback.")
        return str(context.message.author.id)

    print("[DEBUG FINDUSER] Usuario no encontrado")
    return ""



import xfox
import Amisynth.utils as utils
import re

@xfox.addfunc(xfox.funcs)
async def channelPosition(channel_id: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    guild = context.obj_guild
    current_channel = context.obj_channel

    if guild is None:
        print("[DEBUG CHANNELPOSITION] No se puede usar fuera de un servidor.")
        raise ValueError(":x: No se puede usar fuera de un servidor.")

    # Si no se proporciona canal, usar el actual
    if channel_id is None:
        channel = current_channel
    else:
        channel_id = channel_id.strip()
        
        # Detectar mención tipo <#1234567890>
        mention_match = re.match(r"<#(\d+)>", channel_id)
        if mention_match:
            channel_id = mention_match.group(1)

        if not channel_id.isdigit():
            print("[DEBUG CHANNELPOSITION] ID inválido.")
            return ""

        channel = guild.get_channel(int(channel_id))

    if channel:
        print(f"[DEBUG CHANNELPOSITION] Canal encontrado: {channel.name} con posición {channel.position}")
        return str(channel.position)
    
    print("[DEBUG CHANNELPOSITION] Canal no encontrado")
    return ""



import xfox

@xfox.addfunc(xfox.funcs, name="sum")
async def sum_number(first: str, *numbers: str, **kwargs):
    try:
        total = sum(map(float, (first,) + numbers))
        return str(int(total))
    except ValueError:
        return "Invalid number value"



# Función para HTTP PATCH

import aiohttp
import xfox
import Amisynth.utils as utils


import aiohttp
import xfox
import Amisynth.utils as utils
import ast

# Función para HTTP POST
@xfox.addfunc(xfox.funcs)
async def httpPatch(url: str, json: str, *args, **kwargs):
    """Realiza una solicitud HTTP POST con el cuerpo JSON especificado."""
    try:
        # Usamos ast.literal_eval para analizar de forma segura el JSON
        json = ast.literal_eval(json)  # Convertimos la entrada a un valor literal seguro

        async with aiohttp.ClientSession() as session:
            async with session.patch(url, json=json, headers=utils.http_data["headers"]) as response:
                response.raise_for_status()  # Lanza error si hay un problema
                # Almacenar la respuesta JSON en una variable global
                utils.http_response = await response.json()  # Asumiendo que la respuesta es JSON
                utils.http_status = response.status
                print(f"[DEBUG STATUS CHANGED] VALUE: {utils.http_status}")
                print("[DEBUG HTTPOST] Solicitud POST realizada con éxito.")
                return ""
    except Exception as e:
        return f"Error en $httpPost: {str(e)}"



import xfox

@xfox.addfunc(xfox.funcs)
async def replaceText(text, sample, new, amount=-1, *args, **kwargs):
    """
    Reemplaza todas las ocurrencias de 'sample' por 'new' en el texto, limitando los reemplazos a la cantidad especificada.

    :param text: El texto original en el que se realizarán los reemplazos.
    :param sample: El texto que será reemplazado.
    :param new: El texto que reemplazará a 'sample'.
    :param amount: La cantidad máxima de reemplazos. Por defecto es -1, lo que significa que se reemplazarán todas las ocurrencias.
    :return: El texto con los reemplazos realizados.
    """
    
    # Asegurarnos de que 'amount' sea un número entero
    try:
        amount = int(amount)  # Convertir a entero
    except ValueError:
        raise ValueError(f"Invalid amount value: {amount}. It must be an integer.")

    # Reemplazar las ocurrencias del texto
    if amount == -1:
        # Reemplazar todas las ocurrencias
        return text.replace(sample, new)
    else:
        # Limitar la cantidad de reemplazos
        count = 0
        result = []
        for part in text.split(sample):
            result.append(part)
            if count < amount:
                result.append(new)
                count += 1
        return ''.join(result)





import xfox
from Amisynth.utils import ContextAmisynth

@xfox.addfunc(xfox.funcs)
async def isNumber(value=None, *args, **kwargs):
    if value is None:
        raise ValueError(':x: Error en `$isNumber[?]` argumento vacio.')
    try:
        float(value)
        return "true"
    except:
        return "false"



import xfox
import discord
import Amisynth.utils as utils
import asyncio
from typing import Any


@xfox.addfunc(xfox.funcs, "addCmdReaction")
async def addCmdReaction(*args, **kwargs) -> str:
    """
    Adds reactions to a message for each item in args (expected to be emojis).
    
    Args:
        *args: Any number of arguments (emojis) to add as reactions.
        **kwargs: Additional keyword arguments, if needed.
        
    Returns:
        str: An empty string to indicate the completion of the function or an error message.
    """
    context = utils.ContextAmisynth()
    obj = context.obj_message

    # Ensure args is not empty
    if not args:
        raise ValueError(":x: No emojis provided.")

    try:
        # Add each emoji reaction to the message
        for emoji in args:
            await obj.add_reaction(emoji)
    except Exception as e:
        raise ValueError(f"Error adding reactions: {str(e)}")
    print("[DEBUG ADDCMDREACTION] eactions added successfully ")
    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs, "eventEmojiAdd")
async def emoji_addd(*args, **kwargs):
    
    context = utils.ContextAmisynth()
    emoji = context.emoji_name


    return emoji


import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global

@xfox.addfunc(xfox.funcs)
async def getTextSplitLength(*args, **kwargs):
    global split_storage
    return str(len(split_storage.get("last_split", [])))  # Devuelve la cantidad de elementos



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def username(id: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    if id is None:
        username  = context.username
    else:
        username = context.get_username_by_id(int(id))

    return username



import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global

@xfox.addfunc(xfox.funcs)
async def getTextSplitIndex(value: str, *args, **kwargs):
    global split_storage
    if "last_split" not in split_storage:
        raise ValueError("No text has been split yet")
    try:
        index = split_storage["last_split"].index(value) + 1  # Ajustar para que empiece desde 1
        return str(index)
    except ValueError:
        return "-1"  # Si no se encuentra, devuelve 0



import json
import Amisynth.utils as utils
import xfox

@xfox.addfunc(xfox.funcs)
async def httpResult(parametro: str, *args, **kwargs):
    """Devuelve el valor de un parámetro específico dentro del JSON almacenado en utils.http_response.
       Si se pasa '-1', devuelve todo el JSON."""

    # Verificación de la existencia de http_response en utils
    if not hasattr(utils, 'http_response') or utils.http_response is None:
        print("[DEBUG HTTPRESULT] Error: No se ha realizado una solicitud GET previamente.")
        return "Error: No se ha realizado una solicitud GET previamente."

    try:
        # Si la respuesta es una cadena, intentar cargarla como JSON
        if isinstance(utils.http_response, str):
            data = json.loads(utils.http_response)  # Convertimos la cadena a un diccionario
        else:
            data = utils.http_response  # Asumimos que ya es un diccionario

        # Si el parámetro es '-1', devolver todo el JSON de forma legible
        if parametro == "-1":
            return json.dumps(data, indent=2)

        # Devolver el valor correspondiente a la clave solicitada
        if parametro in data:
            return data[parametro]
        else:
            print(f"[DEBUG HTTPRESULT] Error: La clave '{parametro}' no existe en la respuesta.")
            return f"Error: La clave '{parametro}' no se encuentra en la respuesta."

    except json.JSONDecodeError:
        print("[DEBUG HTTPRESULT] Error: No se pudo procesar el JSON.")
        raise ValueError(":x: Error: No se pudo procesar la respuesta JSON.")
    except Exception as e:
        # Manejo de otros posibles errores
        print(f"[DEBUG HTTPRESULT] Error desconocido: {str(e)}")
        raise ValueError(f":x: Error desconocido: {str(e)}")



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArray(*claves, **kwargs):
    if not claves:
        raise ValueError("Error: Debes proporcionar al menos una clave en $jsonArray[]")
    
    data = json_storage
    
    try:
        for clave in claves[:-1]:
            if clave not in data or not isinstance(data[clave], dict):
                data[clave] = {}  # Crea el nivel si no existe
            data = data[clave]  # Avanza en la estructura del JSON
        
        ultima_clave = claves[-1]
        if ultima_clave not in data or not isinstance(data[ultima_clave], list):
            data[ultima_clave] = []  # Inserta el array vacío solo si no existe
        
        return ""
    except Exception as e:
        raise ValueError(f"Error al insertar array en JSON: {e}")




import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def setPrefix(prefix, guild_id:int=None, *args, **kwargs):
    context = utils.ContextAmisynth()
    selft = utils.bot_inst
    if prefix is None:

        raise ValueError(f"Especifica el prefix, `$setPrefix[?;..]` vacio.")
    
    if guild_id is None:
        guild_id=context.guild_id
    selft.set_prefijo_servidor(guild_id, prefix)
    return "" 


import xfox


@xfox.addfunc(xfox.funcs, name="round")
async def round_number(number: str, decimals: str, **kwargs):
    try:
        num = float(number)
        dec = int(decimals)
        return str(round(num, dec))
    except ValueError:
        return ":x: Invalid number value en `$round[]`"



import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def channelName(channel_id: str, *args, **kwargs):
    context = utils.ContextAmisynth()
    guild = context.obj_guild

    if guild is None:
        print("[DEBUG CHANNELNAME] No se puede usar fuera de un servidor.")
        raise ValueError(":x: No se puede usar fuera de un servidor.")

    if not channel_id.isdigit():
        print(f"[DEBUG CHANNELNAME] ID inválido: {channel_id}")
        return ""

    channel = guild.get_channel(int(channel_id))
    if channel:
        print(f"[DEBUG CHANNELNAME] Canal encontrado: {channel.name}")
        return channel.name

    print("[DEBUG CHANNELNAME] Canal no encontrado")
    return ""



import xfox
import re

@xfox.addfunc(xfox.funcs)
async def findEmails(text: str, *args, **kwargs):
    # Expresión regular para detectar emails
    email_pattern = r"[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}"
    emails = re.findall(email_pattern, text)
    
    return ', '.join(emails) if emails else ""



import xfox
import Amisynth.utils as utils
from Amisynth.utils import buttons, menu_options
import discord

@xfox.addfunc(xfox.funcs)
async def editSelectMenuOption(
    menu_id: str, 
    label: str, 
    value: str, 
    description: str, 
    default: str = "False", 
    emoji: str = None, 
    message_id: str = None, 
    *args, **kwargs
):
    from discord import SelectOption
    context = utils.ContextAmisynth()

    default = default.lower() == "true"  # Convertir a booleano
    options = menu_options.get(menu_id, [])

    # Buscar opción por 'value'
    updated = False
    for i, option in enumerate(options):
        if option.value == value:
            options[i] = SelectOption(
                label=label,
                value=value,
                description=description,
                default=default,
                emoji=emoji
            )
            updated = True
            break

    if not updated:
        # Si no existe, agregar la nueva opción
        options.append(SelectOption(
            label=label,
            value=value,
            description=description,
            default=default,
            emoji=emoji
        ))

    # Actualizar el menú guardado
    menu_options[menu_id] = options

    if message_id:
        message = await context.get_message_from_id(int(message_id))

        view = discord.ui.View() if not message.components else discord.ui.View.from_message(message)
        
        # Eliminar cualquier menú con el mismo ID
        view.children = [child for child in view.children if not (isinstance(child, discord.ui.Select) and child.custom_id == menu_id)]

        # Crear nuevo menú actualizado
        select_menu = discord.ui.Select(
            placeholder="Seleccione una opción",
            min_values=1,
            max_values=1,
            options=options,
            custom_id=menu_id
        )
        view.add_item(select_menu)

        await message.edit(view=view)

    return ""



import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def getMessageData(message_id=None, data_type="message", prop="content", embed_index=1, *args, **kwargs):
    context = utils.ContextAmisynth()

    if message_id is None:
        raise ValueError(":x: El argumento `message_id` está vacío en `$getMessageData[]`.")

    message = await context.get_message_from_id(message_id=message_id)
    if message is None:
        raise ValueError(":x: No se encontró ningún mensaje con el ID proporcionado.")

    if data_type.lower() == "message":
        if prop == "content":
            return message.content
        elif prop == "authorID":
            return str(message.author.id)
        elif prop == "username":
            return message.author.name
        elif prop == "avatar":
            return message.author.avatar.url if message.author.avatar else None
        else:
            raise ValueError(f":x: Propiedad de mensaje no válida: `{prop}`")
        
    elif data_type.lower() == "embed":
        # Ajustar el índice para comenzar desde 1
        if int(embed_index) < 1:
            raise ValueError(":x: El índice de embed debe ser mayor o igual a 1.")

        embeds = message.embeds
        if len(embeds) < int(embed_index):
            raise ValueError(":x: Índice de embed inválido o no hay suficientes embeds en el mensaje, error en `$getMessageData[]`.")

        embed = embeds[int(embed_index) - 1]  # Convertir el índice de 1 a 0

        if prop == "title":
            return embed.title
        elif prop == "description":
            return embed.description
        elif prop == "footer":
            return embed.footer.text if embed.footer else None
        elif prop == "color":
            return str(embed.color)
        elif prop == "timestamp":
            return str(embed.timestamp)
        elif prop == "image":
            return embed.image.url if embed.image else None
        else:
            raise ValueError(f":x: Propiedad de embed no válida: `{prop}` en `$getMessageData[{prop};..]`!")
    
    else:
        raise ValueError(f":x: Tipo de dato no válido: `{data_type}` en `$getMessageData[..;{data_type};..]`. Usa `message` o `embed`.")



import xfox

@xfox.addfunc(xfox.funcs)
async def repeatMessage(count: str, message: str, *args, **kwargs):
    try:
        count = int(count)
        if count <= 0:
            return ""
        return " ".join([message] * count)
    except ValueError:
        raise ValueError("Error, en argumento 1, repeatMesssage, numero no vlaido.")



import xfox
from Amisynth.utils import json_storage
import ast

@xfox.addfunc(xfox.funcs)
async def jsonParse(entrada: str, *args, **kwargs):
    try:
        nuevo_json = ast.literal_eval(entrada)  # Evalúa la cadena como un diccionario
        if not isinstance(nuevo_json, dict):
            raise ValueError("El JSON evaluado no es un diccionario válido.")

        
        json_storage.clear()  # Limpia el diccionario original
        json_storage.update(nuevo_json)  # Copia los nuevos valores en él
        
  
        return ""
    except (ValueError, SyntaxError) as e:
        raise ValueError(f"Error en Evaluación del JSON: {str(e)}")



import xfox
import urllib.parse

@xfox.addfunc(xfox.funcs)
async def urlValid(texto: str, *args, **kwargs):
    try:
        parsed = urllib.parse.urlparse(texto)
        if all([parsed.scheme, parsed.netloc]):
            return "True"
        else:
            return "False"
    except Exception:
        return "False"



import xfox
import Amisynth.utils as utils
from Amisynth.utils import menu_options
import discord

@xfox.addfunc(xfox.funcs)
async def editSelectMenu(menu_id: str, min_values: int, max_values: int, placeholder: str = None, message_id: str = None, *args, **kwargs):
    from discord.ui import View, Select

    context = utils.ContextAmisynth()

    # Obtener opciones existentes o inicializar
    options = menu_options.get(menu_id, [])

    # Crear el nuevo menú
    select_menu = Select(
        placeholder=placeholder or "Seleccione una opción",
        min_values=int(min_values),
        max_values=int(max_values),
        options=options,
        custom_id=menu_id
    )

    # Si se proporcionó un mensaje, lo actualizamos
    if message_id:
        message = await context.get_message_from_id(int(message_id))
        view = View() if not message.components else View.from_message(message)

        # Eliminar cualquier menú existente con el mismo ID
        view.children = [child for child in view.children if not (isinstance(child, Select) and child.custom_id == menu_id)]

        view.add_item(select_menu)
        await message.edit(view=view)

    print(F"[DEBUG EDITSELECTMENU] Menú '{menu_id}' actualizado correctamente")
    return f""



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def title(texto_title: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda un título en la lista de embeds, con un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param texto_title: El texto que se quiere mostrar como título en el embed.
    :param indice: El índice opcional del embed (posición en la lista).
    """
    if texto_title is None:
        print("[DEBUG TITLE] La funciom $title esta vicia")
        raise ValueError(":x: Error en $title esta vacio")
    # Crear un embed con el texto de la descripción
    # Crear el embed con el título
    embed = {
        "title": texto_title,
        "index": indice  # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el título
    for i, item in enumerate(embeds):
        if item["index"] == indice:
            # Mantener los otros atributos del embed y solo actualizar el título
            embeds[i]["title"] = texto_title
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox

@xfox.addfunc(xfox.funcs)
async def ping(*args, **kwargs):
    from Amisynth.utils import bot_inst
    bot = bot_inst
    if not bot:
        raise ValueError(":x: Error: el objeto `bot` no está disponible en `$ping[]`.")
    
    latency_ms = round(bot.latency * 1000)
    return f"{latency_ms}"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def getLeaderboardValue(var_type=None, var_name=None, sort_type="desc", position=1, return_type=None, *args, **kwargs):
    """Obtiene el valor ordenado de una variable numérica en el leaderboard."""

    if var_type not in ["guilds", "channels", "users", "global", "global_users"]:
        raise ValueError(":x: Error, el `Variable type` debe ser uno de: `guilds, channels, users, global, global_users`.")

    if var_name is None:
        raise ValueError(":x: Error, el `Variable name` está vacío en `$getLeaderboardValue[?;?]`.")

    position = int(position) - 1  # Ajustamos la posición (1-based a 0-based)

    var = utils.VariableManager()
    data = var.data.get(var_type, {})

    leaderboard = []

    # Extraer valores numéricos según el tipo de variable
    if var_type in ["guilds", "channels", "global"]:
        # Datos directos (sin niveles adicionales)
        for key, values in data.items():
            value = values.get(var_name)
            if value is not None and str(value).isdigit():
                leaderboard.append((key, int(value)))

    elif var_type in ["users", "global_users"]:
        # Datos anidados (requieren recorrer niveles)
        for guild_id, users in data.items():
            for user_id, values in users.items():
                value = values.get(var_name)
                if value is not None and str(value).isdigit():
                    leaderboard.append((user_id, int(value)))

    # Verificar si hay datos numéricos
    if not leaderboard:
        return f":x: No hay valores numéricos para `{var_name}` en `{var_type}`."

    # Ordenar el leaderboard
    leaderboard.sort(key=lambda x: x[1], reverse=(sort_type.lower() == "desc"))

    # Verificar que la posición solicitada exista
    if position < 0 or position >= len(leaderboard):
        return f":x: No hay suficientes datos para la posición `{position+1}`."

    # Obtener el resultado en la posición indicada
    key, value = leaderboard[position]

    # Si return_type es None o está vacío, devolvemos "id - valor"
    if return_type is None or return_type == "":
        return f"{key} - {value}"

    # Si el return_type es 'value', devolvemos solo el valor
    elif return_type.lower() == "value":
        return value
    # Si el return_type es 'id', devolvemos solo el id
    elif return_type.lower() == "id":
        return key
    else:
        raise ValueError(f":x: `Return type` inválido. Usa `value` o `id`.")



import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global


@xfox.addfunc(xfox.funcs)
async def joinSplitText(separator: str, *args, **kwargs):
    global split_storage
    if "last_split" not in split_storage:
        raise ValueError("No text has been split yet")
    
    return separator.join(split_storage.get("last_split", []))


import discord
from Amisynth.utils import menu_options
# Diccionario para almacenar las opciones de cada menú por su ID
import xfox

@xfox.addfunc(xfox.funcs)
async def getMenuValueOption(*args, **kwargs):
    """Agrega una opción al menú de selección, almacenándola en una lista por ID."""
    if "ctx_interaction_env" in kwargs:
        interaction = kwargs["ctx_interaction_env"]
        selected_value = interaction.data['values'][0]
        return selected_value
    return ""



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def authorURL(url: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda un autor con ícono en la lista de embeds, con la URL del ícono y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param url: La URL del ícono que se quiere mostrar en el autor del embed.
    :param indice: El índice opcional del embed (posición en la lista).
    """
    if url is None:
        raise ValueError(":x: Error argumento primero de `$authorURL[?;..]` vacio.")
    # Crear el embed con el autor y el ícono
    embed = {
        "author_url": url,  # URL del ícono en el autor
        "index": indice     # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el autor_url
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar el author_url
            embeds[i]["author_url"] = url
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs, name="clear")
async def clear_func(amount: int, user_id: str = None, remove_pinned: bool = False, *args, **kwargs):
    context = utils.ContextAmisynth()
    channel = await context.get_channel(context.channel_id)

    def check(msg):
        if user_id and str(context.author_id) != user_id:
            return False
        if not remove_pinned and msg.pinned:
            return False
        return True
    
    await channel.purge(limit=amount, check=check)
    return ""



import xfox
import json
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonPretty(indent: str = "4", **kwargs):
    try:
        indent = int(indent)  # Convierte el valor de indent a entero
        return json.dumps(json_storage, ensure_ascii=False, indent=indent)
    except ValueError:
        raise ValueError("Error: El valor de indent debe ser un número entero válido.")
    except Exception as e:
        raise ValueError(f"Error al convertir JSON a string: {e}")



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def mentioned(numero: str = None, fallback_to_author: bool = False, *args, **kwargs):
    context = utils.ContextAmisynth()
    mentions = context.get_mentions()  # Obtiene la lista de IDs mencionados

    if not mentions:
        if fallback_to_author:
            return context.author_id
        return None  # No hay menciones y no se solicita fallback

    if numero is "-":
        return mentions  # Retorna todas las menciones si no se especifica un número

    numero = numero.strip()  # Elimina espacios innecesarios

    if numero == "<":
        return mentions[0]  # Primer mencionado
    elif numero == ">":
        return mentions[-1]  # Último mencionado
    elif numero.isdigit():
        index = int(numero) - 1  # Convertir a índice (1 -> 0, 2 -> 1, etc.)
        if 0 <= index < len(mentions):
            return mentions[index]
    
    return None  # Si el número es inválido o está fuera de rango



import xfox

@xfox.addfunc(xfox.funcs)
async def editMessageBefore(*args, **kwargs):
    if "ctx_message_edit_env" in kwargs:
        before = kwargs["ctx_message_edit_env"][0]
        return before.content
    return ""


import xfox

@xfox.addfunc(xfox.funcs)
async def editMessageAfter(*args, **kwargs):
    if "ctx_message_edit_env" in kwargs:
        after = kwargs["ctx_message_edit_env"][1]
        return after.content
    return ""


import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def getLeaderboardPosition(var_type=None, var_name=None, sort_type="desc", user_id=None, *args, **kwargs):
    """Devuelve la posición de un usuario en el leaderboard basado en una variable numérica."""

    if var_type not in ["guilds", "channels", "users", "global", "global_users"]:
        raise ValueError(":x: Error, el `Variable type` debe ser uno de: `guilds, channels, users, global, global_users`.")

    if var_name is None:
        raise ValueError(":x: Error, el `Variable name` está vacío en `$getLeaderboardPosition[?;?]`.")

    if user_id is None:
        context = utils.ContextAmisynth()
        user_id = context.author_id  # Usa la ID del usuario actual si no se proporciona una

    var = utils.VariableManager()
    data = var.data.get(var_type, {})

    leaderboard = []

    # Extraer valores numéricos según el tipo de variable
    if var_type in ["guilds", "channels", "global"]:
        for key, values in data.items():
            value = values.get(var_name)
            if value is not None and str(value).isdigit():
                leaderboard.append((key, int(value)))

    elif var_type in ["users", "global_users"]:
        for guild_id, users in data.items():
            for u_id, values in users.items():
                value = values.get(var_name)
                if value is not None and str(value).isdigit():
                    leaderboard.append((u_id, int(value)))

    # Verificar si hay datos numéricos
    if not leaderboard:

        raise ValueError(f":x: No hay valores numéricos para `{var_name}` en `{var_type}`.")

    # Ordenar el leaderboard
    leaderboard.sort(key=lambda x: x[1], reverse=(sort_type.lower() == "desc"))

    # Buscar la posición del usuario
    for index, (key, _) in enumerate(leaderboard, start=1):
        if str(key) == str(user_id):
            return index  # Retorna la posición 1-based

    raise ValueError(f":x: El usuario `{user_id}` no tiene un valor registrado para `{var_name}` en `{var_type}`.")



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArrayIndex(*args, **kwargs):
    if len(args) < 2:
        return ""  # Si no se pasan suficientes argumentos, devuelve vacío

    *claves, valor = args  # Extrae claves y el valor a buscar
    data = json_storage

    try:
        for clave in claves:
            if clave not in data:
                return "-1"
            data = data[clave]  # Avanza en la estructura del JSON
        
        if isinstance(data, list):
            return str(data.index(valor)) if valor in data else "-1"
        return "-1"
    except Exception as e:
        raise ValueError(f"Error al buscar el índice en el array: {e}")



import xfox

@xfox.addfunc(xfox.funcs)
async def isBoolean(value=None, *args, **kwargs):
    if value is None:
        return "false"

    value_str = str(value).strip().lower()
    
    positive = ["true", "yes", "on", "enable"]
    negative = ["false", "no", "off", "disable"]
    
    return "true" if value_str in positive + negative else "false"



import xfox
import discord
import Amisynth.utils as utils


@xfox.addfunc(xfox.funcs)
async def kick(user_id: int=None, reason="Ninguna", *args, **kwargs):
    contexto = utils.ContextAmisynth()

    try:
        # Validación de tipo de ID
        if not isinstance(user_id, int):
            raise ValueError("El ID debe ser un número entero válido.")

        # Buscar al miembro
        member = await contexto.obj_guild.fetch_member(user_id)

        # Intentar expulsar al miembro
        await member.kick(reason=reason)

        return ""

    except ValueError as ve:
        # Imprimir el error de valor
        return f"❌ Error de valor: {ve}"

    except discord.NotFound:
        # Si no se encuentra al miembro en el servidor
        return "❌ No se encontró al usuario en este servidor en `$kick`"

    except discord.Forbidden:
        # Si no se tienen permisos suficientes
        return "❌ No tengo permisos suficientes para expulsar al usuario en `$kick`."

    except discord.HTTPException as e:
        # Si ocurre un error HTTP al intentar la acción
        return f"❌ Error de red al expulsar: {e}"

    except Exception as e:
        # Para cualquier otro error inesperado
        return f"❌ Ocurrió un error inesperado: {e}"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs, name="mentionedRoles")
async def mentioned_roles(numero: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    roles = context.get_role_mentions()  # Obtener menciones de roles

    if not roles:  # Verificar si hay menciones de roles
        print("[DEBUG MENTIONED_ROLES]: No se encontraron menciones de roles")
        return ""

    # Si no se proporciona un índice o selector, devolver el primer rol mencionado
    if numero is None:
        return str(roles[0])  # Retorna la mención del primer rol

    # Si el argumento es un número, obtener la mención en ese índice
    if numero.isdigit():
        indice = int(numero) - 1  # Convertir a índice basado en 1
        if 0 <= indice < len(roles):
            return str(roles[indice])  # Retorna la mención del rol en ese índice
        else:
            print("[DEBUG MENTIONED_ROLES]: No hay suficiente cantidad de roles mencionados.")
            return ""

    # Mayor y menor ID de rol
    if numero == ">":
        return str(max(roles, key=lambda role: role.id))  # Mayor ID
    
    if numero == "<":
        return str(min(roles, key=lambda role: role.id))  # Menor ID
    
    print(f"[DEBUG MENTIONED_ROLES]: Parámetro no válido: {numero}")
    raise ValueError(f":x: No pusiste el parámetro adecuado: `{numero}`, en `$mentionedRoles[{numero}]`")



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def author(texto: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda un autor en la lista de embeds, con el texto del autor y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param texto: El texto que se quiere mostrar como autor en el embed.
    :param indice: El índice opcional del embed (posición en la lista).
    """
    if texto is None:
        raise ValueError(f":x: Error el texto de author esta vacio")
    # Crear el embed con el texto del autor
    embed = {
        "author": texto,  # Solo el texto del autor
        "index": indice   # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el texto del autor
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar el texto del autor
            embeds[i]["author"] = texto
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def channelID(nombre: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    if nombre is None:
        channel  = context.channel_id
    else:
        channel = context.obj_guild.fetch_channel(int(nombre))

    return channel



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def userBadges(user_id: int = None, separator: str = "\n", *args, **kwargs):
    contexto = utils.ContextAmisynth()

    resolved_user_id = user_id if user_id is not None else contexto.author_id

    # Intenta buscar al miembro primero
    member = contexto.obj_guild.get_member(resolved_user_id)
    if member is None:
        member = await contexto.obj_guild.fetch_member(resolved_user_id)

    # Accede al objeto User (para obtener public_flags)
    user = member._user if hasattr(member, "_user") else member

    # Lista de posibles flags (badges)
    flags = user.public_flags

    badge_map = {
        "staff": "Discord Staff",
        "partner": "Partner",
        "hypesquad": "HypeSquad Events",
        "bug_hunter": "Bug Hunter",
        "bug_hunter_level_2": "Bug Hunter Level 2",
        "hypesquad_bravery": "Bravery",
        "hypesquad_brilliance": "Brilliance",
        "hypesquad_balance": "Balance",
        "early_supporter": "Early Supporter",
        "verified_bot_developer": "Verified Bot Dev",
        "active_developer": "Active Developer",
        "discord_certified_moderator": "Certified Moderator",
        "premium_promo_dismissed": "Promo Dismissed",
    }

    badges = [name for attr, name in badge_map.items() if getattr(flags, attr, False)]

    return separator.join(badges) if badges else "Este usuario no tiene badges públicas."



import re
import xfox
@xfox.addfunc(xfox.funcs, name="and")
async def andCondition(*conditions: str, **kwargs):
    operadores_validos = ["!=", "==", ">=", "<=", ">", "<"]
    
    def evaluar_condicion(condition):
        patron = r'^(.*?)(!=|==|>=|<=|>|<)(.*?)$'
        coincidencia = re.match(patron, condition)
        
        if not coincidencia:
            return False  # Retorna False si la condición no es válida
        
        izquierda, operador, derecha = coincidencia.groups()
        izquierda, derecha = izquierda.strip(), derecha.strip()
        
        if (izquierda == "" and derecha != "") or (derecha == "" and izquierda != ""):
            return operador != "=="
        
        try:
            izquierda = float(izquierda) if izquierda.replace(".", "", 1).isdigit() else izquierda
            derecha = float(derecha) if derecha.replace(".", "", 1).isdigit() else derecha
            resultado = eval(f'"{izquierda}" {operador} "{derecha}"') if isinstance(izquierda, str) or isinstance(derecha, str) else eval(f'{izquierda} {operador} {derecha}')
            return resultado
        except Exception:
            return False
    
    return "True" if all(evaluar_condicion(cond) for cond in conditions) else "False"


import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def ThreadSlowMode(*args, **kwargs):
    """
    Devuelve el número de miembros de un hilo (Thread).
    Si no se proporciona un ID, devuelve el número de miembros del servidor.
    """
    context = utils.ContextAmisynth()  # Asegúrate de que sea async

    return context.slowmode_delay



import xfox
from Amisynth.utils import json_storage


@xfox.addfunc(xfox.funcs)
async def jsonUnSet(*claves, **kwargs):
    if not claves:
        raise ValueError("Error: Debes proporcionar al menos una clave en $jsonUnSet[]")
    
    data = json_storage
    
    try:
        for clave in claves[:-1]:
            if clave not in data or not isinstance(data[clave], dict):
                return ""
            data = data[clave]
        
        del data[claves[-1]]
        return ""
    except (KeyError, TypeError):
        raise ValueError("Error: No se pudo eliminar la clave en el JSON almacenado")


import xfox

@xfox.addfunc(xfox.funcs, name="abs")
async def absolute_value(*args, **kwargs):
    if not args:
       raise ValueError(":x: Falta un número en `$abs[]`")

    try:
        num = float(args[0])
        return str(abs(num))
    except ValueError:
        return ":x: Valor no numérico en `$abs[]`"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def getVar(nombre:str=None, user_id:str=None, *args, **kwargs):
    """Obtiene una variable, ya sea global de usuario o general, según si se proporciona un user_id."""
    context = utils.ContextAmisynth()
    
    if nombre is None:
        raise ValueError(f":x: Error, el nombre está vacío en la función `$getVar[?]`.")
    
    var = utils.VariableManager()

    if user_id is not None:
        # Si se proporciona un user_id, la variable es global de usuario
        value = var.get_value("global_user", key=nombre, user_id=user_id)
        
        if value is None:
            raise ValueError(f":x: No se encontró la variable `{nombre}` para el usuario especificado.")
        return value

    # Si no se proporciona un user_id, la variable es global
    value = var.get_value("global", key=nombre)
    
    if value is None:
        raise ValueError(f":x: No se encontró la variable `{nombre}` de forma global.")
    
    return value


import xfox
from Amisynth.utils import ContextAmisynth

@xfox.addfunc(xfox.funcs)
async def isHoisted(user_id=None, *args, **kwargs):
    context = ContextAmisynth()
    guild = context.obj_guild
    if user_id is None:
        return "false"
    
    member = await guild.fetch_member(int(user_id))
    for role in member.roles:
        if role.hoist:
            return "true"
    return "false"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs, name="mentionedChannels")
async def mentioned_channels(numero: str = None, fallback_to_current=False, *args, **kwargs):
    context = utils.ContextAmisynth()
    channels = context.get_channel_mentions()  # Obtener menciones de canales

    if not channels:
        if fallback_to_current or fallback_to_current is "true":
            print("[DEBUG MENTIONEDCHANNELS]: No se encontraron menciones, devolviendo canal actual.")
            return str(context.channel_id)
        print("[DEBUG MENTIONEDCHANNELS]: No se encontraron menciones de canales.")
        return ""

    if numero is None:
        return str(channels[0])

    if numero.isdigit():
        indice = int(numero) - 1
        if 0 <= indice < len(channels):
            return str(channels[indice])
        else:
            print("[DEBUG MENTIONED_CHANNELS]: No hay suficiente cantidad de canales mencionados.")
            return ""

    if numero == ">":
        return str(max(channels, key=lambda channel: channel.id))
    
    if numero == "<":
        return str(min(channels, key=lambda channel: channel.id))
    
    print(f"[DEBUG MENTIONED_CHANNELS]: Parámetro no válido: {numero}")
    raise ValueError(f":x: No pusiste el parámetro adecuado: `{numero}`, en `$mentionedChannels[{numero}]`")



import xfox
from Amisynth.utils import embeds  # Asegúrate de que 'embeds' sea la lista global que deseas modificar

@xfox.addfunc(xfox.funcs)
async def image(url_imagen: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda una imagen en la lista de embeds, con una URL de imagen específica y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    """
    if url_imagen is None:
        raise ValueError(":x: Error argumento primero de `$image[?;..]` vacio")
    embed = {
        "image": url_imagen,  # Solo la URL de la imagen
        "index": indice       # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo la imagen
    found = False
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar la imagen
            embeds[i]["image"] = url_imagen
            found = True
            break
    if not found:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def ThreadMembersCount(id: str = None, *args, **kwargs):
    """
    Devuelve el número de miembros de un hilo (Thread).
    Si no se proporciona un ID, devuelve el número de miembros del servidor.
    """
    context = utils.ContextAmisynth()  # Asegúrate de que sea async
    obj_bot = utils.bot_inst

    # Si no se proporciona un ID, obtener el número de miembros del servidor
    if id is None:
        try: 
            count = context.member_count
        except AttributeError:
            return None
    else:
        # Verificar si el ID proporcionado es un canal válido
        obj_thread = await obj_bot.fetch_channel(id)

        if isinstance(obj_thread, discord.Thread):
            fetched_members = await obj_thread.fetch_members()
            count = len(fetched_members.members if hasattr(fetched_members, "members") else fetched_members)
        else:
            return 0  # Podrías lanzar una excepción aquí si prefieres

    return count



import xfox
import discord
import datetime
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs, name="sendEmbedMessage")
async def etc(channel_id=None, 
              content="",
              title=None, 
              title_url=None, 
              description=None,
              color=None,
              author=None, 
              author_icon=None, 
              author_url=None,
              footer=None, 
              footer_icon=None,
              image=None,
              thumbnail=None,
              timestamp=None,
              retorna_id=None,
              *args, **kwargs):
    
    context = utils.ContextAmisynth()

    # Obtener el canal de envío
    if channel_id is None:
        if not hasattr(context, "channel_id") or context.channel_id is None:
            raise ValueError(":x: No se encontró un canal válido en el contexto.")
        channel = await context.get_channel(int(context.channel_id))
    else:
        channel = await context.get_channel(int(channel_id))

    # Crear embed si hay contenido
    embed = None if all(v is None for v in [title, description, color, author, author_icon, author_url, footer, footer_icon, image, thumbnail, timestamp]) else discord.Embed()

    if embed:
        if title:
            embed.title = title
        if title_url:
            embed.url = title_url
        if description:
            embed.description = description
        if color:
            try:
                color = color.lstrip("#")  # Elimina el '#' si existe
                embed.color = int(color, 16)
            except ValueError:
                print(f"[DEBUG COLOR ERROR] El color '{color}' no es un valor hexadecimal válido.")
        if author:
            embed.set_author(name=author, url=author_url or "", icon_url=author_icon or "")
        if footer:
            embed.set_footer(text=footer, icon_url=footer_icon or "")
        if image:
            embed.set_image(url=image)
        if thumbnail:
            embed.set_thumbnail(url=thumbnail)
        if timestamp:
            if timestamp == "true":
                embed.timestamp = datetime.datetime.utcnow()
            elif timestamp != "false":
                print(f"[DEBUG TIMESTAMP ERROR] Argumento no válido: `{timestamp}` en `$sendEmbedMessage[]`, usa true/false.")
                raise ValueError(f":x: Argumento no válido: `{timestamp}` en `$sendEmbedMessage[]`, usa true/false.")

    # Agregar fields desde *args
    args_list = list(args)
    for i in range(0, len(args_list), 3):
        try:
            name = args_list[i]
            value = args_list[i + 1]
            inline = args_list[i + 2] if i + 2 < len(args_list) else "true"
            inline = inline.lower() == "true"
            embed.add_field(name=name, value=value, inline=inline)
        except IndexError:
            missing_parts = ["nombre", "valor", "inline"][len(args_list) % 3:]
            print(f"[DEBUG FIELD ERROR] Faltan: {', '.join(missing_parts)} en `$sendEmbedMessage`.")
            raise ValueError(f":x: Faltan valores en un field: {', '.join(missing_parts)}.")

    # Enviar el mensaje
    message = await channel.send(content, embed=embed)

    # Retornar ID si se requiere
    if retorna_id:
        if retorna_id == "true":
            return message.id
        elif retorna_id != "false":
            print(f"[DEBUG RETORNAR_ID ERROR] Argumento no válido: `{retorna_id}` en `$sendEmbedMessage[]`, usa true/false.")
            raise ValueError(f":x: Argumento no válido: `{retorna_id}` en `$sendEmbedMessage[]`, usa true/false.")

    print("[DEBUG AMYSINTH ERROR] Contacta con Soporte: https://discord.gg/NyGuP3e5")
    return ""



import aiohttp
import xfox
import Amisynth.utils as utils
import ast

# Función para HTTP POST
@xfox.addfunc(xfox.funcs, name="httpStatus")
async def http_status(*args, **kwargs):
    return utils.http_status


import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def color(color: str=None, indice: int = 1, *args, **kwargs):
    """
    Guarda un color en la lista de embeds, con un color específico y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param color: El color en formato hexadecimal (con o sin '#').
    :param indice: El índice opcional del embed (posición en la lista).
    """

    if color is None:
        raise ValueError(":x: Error argumento color vacio. `$color[?;..]`")
    # Asegurarse de que el color tiene el formato adecuado
    if color.startswith('#'):
        color = color[1:]  # Eliminar el '#' si está presente
    if len(color) == 6:  # Asegurarse de que el código hexadecimal sea válido
        try:
            int(color, 16)  # Comprobar si el color es un número hexadecimal
        except ValueError:
            raise ValueError(f"Error: El color proporcionado no es un código hexadecimal válido: $color[{color}]")
    else:
        raise ValueError(f"Error El color debe ser un código hexadecimal de 6 caracteres.")

    # Crear el embed con solo el color
    embed = {
        "color": int(color, 16),  # Convertir el código hexadecimal a entero
        "index": indice           # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el color
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar el color
            embeds[i]["color"] = int(color, 16)
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def setServerVar(nombre=None, value=None, guild_id=None, *args, **kwargs):
    """Establece una variable para un servidor específico."""
    context = utils.ContextAmisynth()

    if nombre is None or value is None:
        raise ValueError(":x: Error, el nombre o el valor están vacíos en la función `$setServerVar[?;?]`.")
    
    if guild_id is None:
        guild_id = context.guild_id

    var = utils.VariableManager()
    
    # Establece el valor para el servidor especificado
    var.set_value("guild", key=nombre, value=value, guild_id=guild_id)
    return ""


import xfox
import discord
import Amisynth.utils as utils


@xfox.addfunc(xfox.funcs)
async def ban(user_id: int, reason="Sin razón específica", *args, **kwargs):
    contexto = utils.ContextAmisynth()

    try:
        # Validación del ID
        if not isinstance(user_id, int):
            raise ValueError("El ID debe ser un número entero válido.")

        # Buscar al miembro
        member = await contexto.obj_guild.fetch_member(user_id)

        if member is None:
            raise discord.NotFound("No se encontró al usuario en el servidor.")

        # Banear al miembro
        await member.ban(reason=reason)

        return f"✅ Usuario `{member}` ha sido baneado. Razón: `{reason}`"

    except ValueError as ve:
        return f"❌ Error de valor: {ve}"

    except discord.NotFound:
        return "❌ No se encontró al usuario en este servidor."

    except discord.Forbidden:
        return "❌ No tengo permisos suficientes para banear al usuario."

    except discord.HTTPException as e:
        return f"❌ Error de red al banear: {e}"

    except Exception as e:
        return f"❌ Ocurrió un error inesperado: {e}"



import aiohttp
import json
import Amisynth.utils as utils  # Importamos el archivo utils donde está http_data
import xfox

# Función para hacer la solicitud GET
@xfox.addfunc(xfox.funcs)
async def httpGet(url: str, *args, **kwargs):
    """Hace una solicitud HTTP GET con las cabeceras personalizadas y almacena el contenido en una variable global."""
    global utils  # Usamos el diccionario http_data de utils
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(url, headers=utils.http_data["headers"]) as response:
                response.raise_for_status()
                utils.http_response = await response.text()  # Almacena la respuesta
                utils.http_status = await response.status
                print("[DEBUG HTTPGET] Respuesta GET almacenada.")
                return ""
    except Exception as e:
        return f"Error en $httpGet: {str(e)}"



import xfox
import Amisynth.utils as utils
import re

@xfox.addfunc(xfox.funcs)
async def findRole(query: str, *args, **kwargs):
    context = utils.ContextAmisynth()
    guild = context.obj_guild

    if guild is None:
        print("[DEBUG FINDROLE] No se puede buscar fuera de un servidor.")
        raise ValueError(":x: No se puede buscar fuera de un servidor.")

    query = query.strip()

    # 🟦 Buscar por mención de rol <@&123456789012345678>
    mention_match = re.match(r"<@&(\d+)>", query)
    if mention_match:
        role_id = int(mention_match.group(1))
        role = guild.get_role(role_id)
        if role:
            print(f"[DEBUG FINDROLE] Encontrado por mención: {role}")
            return str(role.id)

    # 🟩 Buscar por ID
    if query.isdigit():
        role = guild.get_role(int(query))
        if role:
            print(f"[DEBUG FINDROLE] Encontrado por ID: {role}")
            return str(role.id)

    # 🟨 Buscar por nombre
    query_lower = query.lower()
    for role in guild.roles:
        if role.name.lower() == query_lower:
            print(f"[DEBUG FINDROLE] Encontrado por nombre: {role}")
            return str(role.id)

    print("[DEBUG FINDROLE] Rol no encontrado")
    return ""



import xfox
from Amisynth.Functions.storage import split_storage  # Importamos el almacenamiento global

@xfox.addfunc(xfox.funcs)
async def splitText(index: str, *args, **kwargs):
    global split_storage
    try:
        index = int(index) - 1  # Ajustar para que comience desde 1
        return split_storage.get("last_split", [""])[index]  # Obtener el índice solicitado
    except (ValueError, IndexError):
        return ""  # Si el índice no existe, devolver vacío



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArraySort(*claves, **kwargs):
    if not claves:
        return ""  # Si no se pasan claves, devuelve vacío
    
    data = json_storage

    try:
        for clave in claves[:-1]:
            if clave not in data or not isinstance(data[clave], dict):
                return ""  # Si alguna clave intermedia no existe, devuelve vacío
            data = data[clave]  # Avanza en la estructura del JSON
        
        last_key = claves[-1]

        if last_key not in data or not isinstance(data[last_key], list):
            return ""  # Si la clave final no existe o no es un array, devuelve vacío

        data[last_key].sort()  # Ordena el array en orden ascendente
        return ""
    except Exception as e:
        raise ValueError(f"Error al ordenar el array: {e}")



import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs, name="addField")
async def add_field(nombre: str=None, valor: str=None, inline: bool = False, indice: int = 1, *args, **kwargs):
    """
    Agrega un campo (field) a un embed en la lista de embeds.
    Si el embed en el índice especificado ya existe, se agrega el field a su lista de fields.
    Si no existe, se crea un nuevo embed con ese índice y el field.
    
    :param nombre: Nombre del campo (name).
    :param valor: Contenido del campo (value).
    :param inline: Si el campo es inline o no (por defecto, True).
    :param indice: Índice del embed en la lista (por defecto, 1).
    """
    if nombre is None:
        raise ValueError(":x: Error argumento primero de `$addField[?;..]` vacio")
    
    if valor is None:
        raise ValueError(":x: Error argumento segundo de `$addField[..;?]` vacio")
    

    # Buscar si ya existe un embed con ese índice
    for i, item in enumerate(embeds):
        if item["index"] == indice:
            # Si el embed no tiene fields, inicializar la lista
            if "fields" not in embeds[i]:
                embeds[i]["fields"] = []
            
            # Agregar el nuevo campo
            embeds[i]["fields"].append({
                "name": nombre,
                "value": valor,
                "inline": inline
            })
            break
    else:
        # Si no se encontró, crear un nuevo embed con el field
        embed = {
            "index": indice,
            "fields": [{
                "name": nombre,
                "value": valor,
                "inline": inline
            }]
        }
        embeds.append(embed)

    return ""



import xfox

@xfox.addfunc(xfox.funcs, name="multi")
async def multiply_numbers(first: str, *numbers: str, **kwargs):
    try:
        result = float(first)
    except ValueError:
        raise ValueError(f":x: Invalid number value: '{first}' en `$multi[]`")

    for n in numbers:
        try:
            result *= float(n)
        except ValueError:
            raise ValueError(f":x: Invalid number value: '{n}' en `$multi[]`")

    return str(int(result)) if result.is_integer() else str(result)



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArrayUnshift(*claves, value=None, **kwargs):
    if not claves:
        return ""  # Si no se pasan claves, devuelve vacío
    
    if value is None:
        return ""  # Si no se proporciona un valor, devuelve vacío

    data = json_storage

    try:
        for clave in claves[:-1]:
            if clave not in data or not isinstance(data[clave], dict):
                return ""  # Si alguna clave intermedia no existe, devuelve vacío
            data = data[clave]  # Avanza en la estructura del JSON
        
        last_key = claves[-1]
        
        if last_key not in data or not isinstance(data[last_key], list):
            data[last_key] = []  # Si la clave no existe o no es un array, la inicializa como lista
        
        data[last_key].insert(0, value)  # Agrega el valor al inicio del array
        return ""
    except Exception as e:
        raise ValueError(f"Error al hacer unshift en el array: {e}")



import xfox
from Amisynth.utils import json_storage
import ast

@xfox.addfunc(xfox.funcs)
async def jsonSet(*args, **kwargs):
    if len(args) < 2:
        raise ValueError("Debes establecer una clave y valor")
    
    *claves, valor = args
    data = json_storage
    
    try:
        for clave in claves[:-1]:
            if clave not in data or not isinstance(data[clave], dict):
                data[clave] = {}
            data = data[clave]
        
        data[claves[-1]] = valor
        return ""
    except (KeyError, TypeError):
       raise ValueError("Nose pudo establece en JSON Almacenado")


import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def getServerVar(nombre:str=None, guild_id:str=None, *args, **kwargs):
    """Obtiene una variable para un servidor específico."""
    context = kwargs["ctx" ]
    
    if nombre is None:
        raise ValueError(":x: Error, el nombre está vacío en la función `$getServerVar[?]`.")
    
    if guild_id is None:
        guild_id = context.guild_id or context.guild.id

    var = utils.VariableManager()

    # Obtiene el valor para el servidor especificado
    value = var.get_value("guild", key=nombre, guild_id=guild_id)
    
    if value is None:
        return f":x: No se encontró la variable `{nombre}` para el servidor especificado."
    
    return value


import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def unban(user_id: int, *args, **kwargs):
    contexto = utils.ContextAmisynth()

    try:
        # Validación del ID
        if not isinstance(user_id, int):
            raise ValueError("El ID debe ser un número entero válido.")

        # Intentar obtener el usuario (por ID) en la lista de baneos
        banned_users = await contexto.obj_guild.bans()
        member_to_unban = discord.utils.get(banned_users, user__id=user_id)

        if member_to_unban is None:
            raise discord.NotFound("No se encontró al usuario en la lista de baneos.")

        # Unbanear al usuario
        await contexto.obj_guild.unban(member_to_unban.user)

        return f"✅ Usuario `{member_to_unban.user}` ha sido desbaneado."

    except ValueError as ve:
        return f"❌ Error de valor: {ve}"

    except discord.NotFound:
        return "❌ El usuario no está baneado en este servidor."

    except discord.Forbidden:
        return "❌ No tengo permisos suficientes para desbanear al usuario."

    except discord.HTTPException as e:
        return f"❌ Error de red al desbanear: {e}"

    except Exception as e:
        return f"❌ Ocurrió un error inesperado: {e}"



import os
import importlib.util


def load_functions():
    """Carga automáticamente todas las funciones de la carpeta 'Functions'."""
    base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))  
    folder = os.path.join(base_path, "Functions")  # Accede a `Amisynth/Functions`

    if not os.path.exists(folder):
        print(f"⚠️ No se encontró la carpeta 'Functions' en {folder}")
        return  

    for root, dirs, files in os.walk(folder):
        for filename in files:
            if filename.endswith(".py"):
                module_name = filename[:-3]  # Nombre del módulo sin `.py`
                module_path = os.path.join(root, filename)  # Ruta completa del archivo

                spec = importlib.util.spec_from_file_location(module_name, module_path)
                module = importlib.util.module_from_spec(spec)

                try:
                    spec.loader.exec_module(module)  # Carga y ejecuta el módulo sin almacenarlo
                    # print(f"✅ Módulo cargado: {module_name}")
                    print(f"✅ Módulo cargado: {module_name}")
                except Exception as e:
                    print(f"❌ Error al cargar {module_name}: {e}")




import xfox
from Amisynth.utils import embeds  # Asumo que embeds es una lista global que estás usando

@xfox.addfunc(xfox.funcs)
async def titleURL(url: str, indice: int = 1, *args, **kwargs):
    """
    Guarda una URL en el título del embed, con un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    
    :param url: La URL que se quiere asociar al título del embed.
    :param indice: El índice opcional del embed (posición en la lista).
    """

    if url is None:
        print("[DEBUG TITLE] La funciom $titleURL[?;..] esta vicia")
        raise ValueError(":x: Error en `$titleURL[?;..]` esta vacio")
    # Crear el embed con la URL en el título
    embed = {
        "title_url": url,  # URL asociada al título
        "index": indice    # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo la URL
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar la URL
            embeds[i]["title_url"] = url
            break
    else:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import xfox
import urllib.parse

@xfox.addfunc(xfox.funcs)
async def url(modo: str, texto: str, *args, **kwargs):
    if modo.lower() == "encode":
        return urllib.parse.quote_plus(texto)
    elif modo.lower() == "decode":
        return urllib.parse.unquote_plus(texto)
    else:
        raise ValueError(f"Modo inválido: '{modo}'. Usa 'encode' o 'decode', en `$url[{modo};..]`")



import xfox

@xfox.addfunc(xfox.funcs, name="average")
async def average_of_numbers(first: str, *numbers: str, **kwargs):
    try:
        numbers = [float(first)] + [float(n) for n in numbers]
        return str(sum(numbers) / len(numbers))
    except ValueError:
        return ":x: Invalid number value en `$average[]`"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def setVar(nombre=None, value=None, user_id=None, *args, **kwargs):
    """Establece una variable, ya sea global de usuario o general, según si se proporciona un user_id."""
    context = utils.ContextAmisynth()

    if nombre is None or value is None:
        raise ValueError(":x: Error, el nombre o el valor están vacíos en la función `$setVar[?;?]`.")
    
    var = utils.VariableManager()

    if user_id is not None:
        # Si se proporciona un user_id, la variable es global de usuario
        var.set_value("global_user", key=nombre, value=value, user_id=user_id)
        print("[DEBUG SETVAR] Variable `{nombre}` establecida con éxito como global de usuario.")
        return ""

    # Si no se proporciona un user_id, la variable es general
    var.set_value("global", key=nombre, value=value)
    print("[DEBUG SETVAR] Variable `{nombre}` establecida con éxito como global.")
    return ""


from typing import List, Dict, Optional
import xfox
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def messageSlash(nombre: str, *args, **kwargs):
    context = utils.ContextAmisynth()
    datos = context.slash_options
    
    if datos is None:
        return ""

    for item in datos:
        if item.get("name") == nombre:
            return item.get("value")  # Retorna el valor si encuentra coincidencia
    return ""



import xfox
import Amisynth.utils as utils
import discord

@xfox.addfunc(xfox.funcs)
async def newModal(id, nombre, *args, **kwargs):
    # Usamos los componentes que tienes en utils.modals (que asumo son discord.ui.TextInput)
    print(f"[DEBUG NEWMODAL] Parámetros recibidos: id={id}, nombre={nombre}, args={args}, kwargs={kwargs}")
    
    n = utils.modals
    print(f"[DEBUG NEWMODAL] Lista de modales: {n}")
    
    # Creamos el modal usando la lista de componentes
    view = discord.ui.Modal(title=nombre, custom_id=id)
    print(f"[DEBUG NEWMODAL] Modal creado: {view}")

    for i in n:
        view.add_item(i)
        print(f"[DEBUG NEWMODAL] Componente agregado al modal: {i}")

    # Definir lo que ocurre cuando el modal se envíe
    async def on_submit(interaction: discord.Interaction):
        # Extraer los valores de los componentes
        values = {}
        print(f"[DEBUG NEWMODAL] Enviando valores del modal: {values}")
        for child in view.children:
            values[child.custom_id] = child.value or "No especificado"
            print(f"[DEBUG NEWMODAL] Valor de {child.custom_id}: {child.value}")
        
        # Crear el mensaje con los resultados
        response_message = "\n".join([f"{key}: {value}" for key, value in values.items()])
        print(f"[DEBUG NEWMODAL] Mensaje a enviar: {response_message}")

        # Enviar el mensaje como respuesta
        await interaction.response.send_message(response_message, ephemeral=True)
        print("[DEBUG NEWMODAL] Mensaje enviado como respuesta.")

    # Asignar la función on_submit al modal
    view.on_submit = on_submit
    print(f"[DEBUG NEWMODAL] Función on_submit asignada al modal.")

    # Actualizar los modales en utils (si es necesario para mantener el estado)
    utils.modals = view
    print(f"[DEBUG NEWMODAL] Modal actualizado en utils.modals.")

    # Retornar vacío, ya que el modal se maneja internamente
    return ""



import xfox
import discord
import Amisynth.utils as utils


@xfox.addfunc(xfox.funcs)
async def userExists(user_id: int, *args, **kwargs):
    contexto = utils.ContextAmisynth()  # Suponiendo que tienes una forma de obtener el contexto de la guild
    try:
        # Intenta obtener al miembro por su ID
        member = await contexto.obj_guild.fetch_member(user_id)
        return "True"  # Si el miembro existe, devuelve True
    except discord.NotFound:
        return "False"  # Si el miembro no es encontrado, devuelve False
    except discord.HTTPException as e:
        raise ValueError(f":x: Error al buscar usuario: {str(e)}")



import xfox
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def message(index: str, *args, **kwargs):
    contexto = utils.ContextAmisynth()
    
    argumentos = contexto.arguments or []  # Asegurar que sea una lista


    try:
        index = int(index)  # Convertir una sola vez
    except ValueError:
        return ""  # Si no es un número, devolver vacío

    if index == -1:
        return " ".join(argumentos)  # Unir toda la lista en una cadena
    if 1 <= index <= len(argumentos):
        return argumentos[index - 1]  # Obtener el elemento ajustando índice
    return ""  # Si el índice está fuera de rango



import xfox
import Amisynth.utils as utils
from discord.ui import TextInput, TextInputStyle

@xfox.addfunc(xfox.funcs)
async def addTextInput(
    input_id: str,
    style: str,
    label: str,
    min_length: int = None,
    max_length: int = None,
    required: str = "yes",  # Acepta 'yes' o 'no'
    value: str = None,
    placeholder: str = None,
    *args, **kwargs
):
    # Validaciones de parámetros
    print(f"[DEBUG ADDTEXTINPUT] Verificando parámetros: label={label}, style={style}, min_length={min_length}, max_length={max_length}, required={required}, value={value}, placeholder={placeholder}")
    
    if len(label) > 45:
        print("[DEBUG ADDTEXTINPUT] Error: El label tiene más de 45 caracteres.")
        return "El label no puede tener más de 45 caracteres."
    
    # Validar estilo: 'short' -> singleline, 'paragraph' -> multiline
    if style not in ["short", "paragraph"]:
        print(f"[DEBUG ADDTEXTINPUT] Error: El estilo '{style}' no es válido.")
        return "El estilo debe ser 'short' (singleline) o 'paragraph' (multiline)."
    
    # Convertir el estilo a su valor correspondiente
    style_enum = TextInputStyle.singleline if style == "short" else TextInputStyle.multiline
    print(f"[DEBUG ADDTEXTINPUT] Estilo convertido a: {style_enum}")
    
    # Validaciones de longitudes
    if min_length is not None:
        if min_length < 0 or min_length > 4000:
            print(f"[DEBUG ADDTEXTINPUT] Error: Longitud mínima {min_length} fuera de rango.")
            return "La longitud mínima debe estar entre 0 y 4000."
        print(f"[DEBUG ADDTEXTINPUT] Longitud mínima válida: {min_length}")
    
    if max_length is not None:
        if max_length < 0 or max_length > 4000 or (min_length is not None and max_length < min_length):
            print(f"[DEBUG ADDTEXTINPUT] Error: Longitud máxima {max_length} fuera de rango o menor que la mínima.")
            return "La longitud máxima debe estar entre 0 y 4000, y no puede ser menor que la longitud mínima."
        print(f"[DEBUG ADDTEXTINPUT] Longitud máxima válida: {max_length}")
    
    # Validar si es requerido
    if required not in ["yes", "no"]:
        print(f"[DEBUG ADDTEXTINPUT] Error: El parámetro 'Required?' tiene un valor inválido '{required}'.")
        return "El parámetro 'Required?' debe ser 'yes' o 'no'."

    required_bool = True if required == "yes" else False
    print(f"[DEBUG ADDTEXTINPUT] 'Required?' convertido a booleano: {required_bool}")
    
    # Crear el TextInput
    input_box = TextInput(
        custom_id=input_id,
        style=style_enum,
        label=label,
        min_length=min_length,
        max_length=max_length,
        required=required_bool,
        default=value,
        placeholder=placeholder
    )
    
    print(f"[DEBUG ADDTEXTINPUT] Input Box creado: {input_box}")

    # Agregar al modal
    modal = utils.modals
    print(f"[DEBUG ADDTEXTINPUT] Modal antes de agregar el input: {modal}")
    modal.add_item(input_box)
    print(f"[DEBUG ADDTEXTINPUT] Modal después de agregar el input: {modal}")
    
    return f"Input agregado correctamente."



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArrayAppend(*args, **kwargs):
    if len(args) < 2:
        return ""  # Si no se pasan suficientes argumentos, devuelve vacío

    *claves, valor = args  # Extrae claves y el valor a agregar
    data = json_storage

    try:
        for clave in claves:
            if clave not in data:
                data[clave] = {}  # Si la clave no existe, crea un diccionario
            data = data[clave]  # Avanza en la estructura del JSON
        
        if isinstance(data, list):
            data.append(valor)  # Agrega el valor al array
        else:
            data[claves[-1]] = [valor]  # Si no es un array, lo convierte en uno con el valor

        return ""
    except Exception as e:
        raise ValueError(f"Error al agregar el valor al array: {e}")



import xfox

@xfox.addfunc(xfox.funcs, name="divide")
async def dividi_number(first: str, *numbers: str, **kwargs):
    try:
        first_num = float(first)
        for num in numbers:
            divisor = float(num)
            if divisor == 0:
                raise ValueError(":x: Division by zero error")
            first_num /= divisor
        return str(first_num)
    except ValueError:
        raise ValueError("Invalid number value")





import discord
from discord.ext import commands
from discord import app_commands
from discord.ui import View, Button, Select


import asyncio
import xfox
import Amisynth.Handler
import Amisynth.utils as utils
from typing import List, Dict, Optional, Any
import os
import importlib.util


from Amisynth.utils import options_slash
from Amisynth.utils import clear_data

# Registrar todas las funciones automáticamente
Amisynth.Handler.register_all()

class AmiClient(commands.Bot):
    def __init__(self, prefix, cogs=None, variables_json=False, case_insensitive:bool=False):
        super().__init__(command_prefix=prefix, intents=discord.Intents.all(), case_insensitive=case_insensitive)
        
        self.prefix = prefix
        self.servicios_prefijos = {}  # Diccionario para guardar los prefijos por servidor
        self._cogs = cogs or []
        self.comandos_personalizados = {}
        self.eventos_personalizados = {
            "$onMessage": [],
            "$onReady": [],
            "$onReactionAdd": [],
            "$onReactionRemove": [],
            "$onInteraction": [],
            "$onMessageEdit": [],
            "$onMessageDelete": [],
            "$onJoinMember": [],
            "$onLeaveMember": [],
            "$onMessagesPurged": [],
            "$onMessageTyping": [],
            "$onChannelCreate": [],
            "$onChannelDelete": [],
            "$onChannelEdit": [],
            "$onThreadCreate": [],
            "$onThreadRemove": [],
            "$onThreadJoin": [],
            "$onThreadDelete": [],
            "$onThreadUpdate": [],
            "$onThreadJoinMember": [],
            "$onThreadRemoveMember": []
        }

        if variables_json == True:
            utils.VariableManager()

        utils.bot_inst = self

    async def get_prefix(self, msg):
        """Obtiene el prefijo de acuerdo con el servidor o usa el prefijo general."""
        if msg.guild:
            # Si el servidor tiene un prefijo personalizado, lo devuelve
            return self.servicios_prefijos.get(msg.guild.id, self.prefix)
        return self.prefix  # Usa el prefijo general si no es un servidor
    
    def set_prefijo_servidor(self, servidor_id, nuevo_prefijo):
        """Permite cambiar el prefijo para un servidor específico."""
        self.servicios_prefijos[servidor_id] = nuevo_prefijo


    async def setup_hook(self):
        """Cargar todos los cogs de forma asincrónica."""
        if self._cogs:  # Verificar si se pasó una carpeta de cogs
            await self.load_cogs(self._cogs)

    async def load_cogs(self, carpeta):
        """Cargar cogs de forma asincrónica."""
        for filename in os.listdir(carpeta):
            if filename.endswith(".py"):
                cog_path = os.path.join(carpeta, filename)

                # Cargar módulo dinámicamente
                spec = importlib.util.spec_from_file_location(filename[:-3], cog_path)
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)

                # Si el cog tiene una función setup(bot), la ejecutamos
                if hasattr(module, "setup"):
                    await module.setup(self)  # Ejecutar setup(bot) si es async


    def new_command(self, name, type, code):
        async def custom_command(ctx_command):
            utils.ContextAmisynth(ctx=ctx_command)

            try:
              
                result = await xfox.parse(code, del_empty_lines=True)
             
            except ValueError as e:
                result = e
        
            texto = result
            botones, embeds = await utils.utils()
            view = discord.ui.View()
            botones_en_fila = 0
            # Construir el View si hay botones
            for boton in botones:
                if botones_en_fila < 5:
                    view.add_item(boton)
                    botones_en_fila += 1
                else:
            # Si hay más de 5 botones, empieza una nueva fila
                    view.add_item(boton)
                    botones_en_fila = 1
            

            # Enviar mensaje con el tipo adecuado
            try:
                message = await ctx_command.send(
                content=texto if texto else None,  # Si hay texto, se agrega
                view=view if botones else None,    # Si hay botones, se agrega el View
                embeds=embeds if embeds else None  # Si hay embeds, se agregan
                )
                kwargs = {'message': message}
                await clear_data()
                await xfox.parse(code, **kwargs)
                await clear_data()

            except ValueError as e:
                print(f"[ERROR COMMAND]: {e}")
            except discord.HTTPException as e:
                print(f"[ERROR] {e}")

       




        self.comandos_personalizados[name] = {"type": type, "code": code}
        self.add_command(commands.Command(custom_command, name=name))
    
    
    def new_slash(
        self,
        name: str,
        description: str,
        code: str = "",
    ):
        parameters = ["interaction: discord.Interaction"]
        choices_kwargs = {}

        
        options = options_slash
        options = asyncio.run(xfox.parse(code))
        print("[DEBUG NEWSLASH] PARSE COMPLETED")
        options = options_slash
        print("[DEBUG NEWSLASH] OPTIONS:", options_slash)

        def remp(valor):
            if valor.lower() == "texto":
                return "str"  # Devuelve el tipo de texto (string)
            elif valor.lower() == "integer":
                return "int"  # Devuelve el tipo entero
            elif valor.lower() == "archivo":
                return "discord.Attachment"  # Devuelve el tipo de archivo adjunto
            elif valor.lower() == "canal":
                return "discord.TextChannel"  # Devuelve el tipo de canal de texto
            elif valor.lower() == "mencionable":
                return "discord.Member" # Devuelve el tipo de miembro mencionable (puede ser un usuario o rol)
            elif valor.lower() == "rol":
                return "discord.Role"  # Devuelve el tipo de rol
            elif valor.lower() == "numero":
               return "float"  # Devuelve el tipo de número (puede ser entero o flotante)
            else:
               raise ValueError("[DEBUG SLASH] OPCION DE SLASH INVALIDA")
            
        describe_kwargs = {}
        
        if options:
            for option in options:
                option_name = option.get("name_option")
                param_name = option_name.replace(" ", "_")
                option_type = remp(valor=option["type"])
                option_required = option.get("required", False)
                
                if option_required:
                    parameters.append(f"{param_name}: {option_type}")
                else:
                    parameters.append(f"{param_name}: {option_type} = None")

                if "description" in option:
                    describe_kwargs[param_name] = option["description"]

                if "choices" in option and isinstance(option["choices"], list):
                    choices_kwargs[param_name] = [
                        app_commands.Choice(name=choice["name_choice"], value=choice["value_choice"])
                        for choice in option["choices"]
                    ]

        params_str = ", ".join(parameters)

        func_code = f"""async def slash_command({params_str}):
        kwargs = {{"ctx_slash_env": interaction}}
        utils.ContextAmisynth(interaction)

        result = await xfox.parse({repr(code)}, del_empty_lines=True, **kwargs)
        botones, embeds = await utils.utils()
           
        view = discord.ui.View()
        if botones:
            for boton in botones: 
                view.add_item(boton)

        await interaction.response.send_message(
            content=result if result else None,
            view=view,
            embeds=embeds if embeds else [],
            ephemeral=False



        )
        await clear_data()

        
        """
        
        exec(func_code, globals(), locals())
        command_func = locals()["slash_command"]
        # 🔸 Registrar el comando con @self.tree.command(...)

        if describe_kwargs:
            decorated_func = app_commands.describe(**describe_kwargs)(command_func)
        else:
            decorated_func = command_func
        decorated_func = self.tree.command(name=name, description=description)(decorated_func)

        
        for key, choices in choices_kwargs.items():
            decorated_func = app_commands.choices(**{key: choices})(decorated_func)
        
        self.comandos_personalizados[name] = {"type": "slash", "code": code}

        





    def new_event(self, 
                  tipo, 
                  codigo, 
                  overwrite=False):
        
        if tipo not in self.eventos_personalizados or overwrite:
            self.eventos_personalizados[tipo] = []  # Reiniciar si se sobrescribe
        self.eventos_personalizados[tipo].append(codigo)

    async def ejecutar_eventos(self, tipo, 
                               ctx_message_env=None, 
                               ctx_reaction_env=None, 
                               ctx_reaction_remove_env=None, 
                               ctx_interaction_env=None, 
                               ctx_message_edit_env=None, 
                               ctx_message_delete_env=None,
                               ctx_join_member_env=None,
                               ctx_remove_member_env=None,
                               ctx_bulk_message_delete_env=None,
                               ctx_typing_env=None,
                               ctx_guild_channel_create=None,
                               ctx_guild_channel_delete=None,
                               ctx_guild_channel_edit=None,
                               ctx_thread_create_env=None,
                               ctx_thread_remove_env=None,
                               ctx_thread_join_env=None,
                               ctx_thread_delete_env=None,
                               ctx_thread_update_env=None,
                               ctx_thread_member_join_env=None,
                               ctx_thread_member_remove_env=None
                               
                               ):
        

        if tipo in self.eventos_personalizados:
            for codigo in self.eventos_personalizados[tipo]:
                kwargs = {
                    "ctx_message_env": ctx_message_env,
                    "ctx_reaction_env": ctx_reaction_env,
                    "ctx_reaction_remove_env": ctx_reaction_remove_env,
                    "ctx_interaction_env": ctx_interaction_env,  # 👈 Agregado aquí
                    "ctx_message_edit_env": ctx_message_edit_env,
                    "ctx_message_delete_env": ctx_message_delete_env,
                    "ctx_join_member_env": ctx_join_member_env,
                    "ctx_remove_member_env": ctx_remove_member_env,
                    "ctx_bulk_message_delete_env": ctx_bulk_message_delete_env,
                    "ctx_typing_env": ctx_typing_env,
                    "ctx_guild_channel_create": ctx_guild_channel_create,
                    "ctx_guild_channel_delete": ctx_guild_channel_delete,
                    "ctx_guild_channel_edit": ctx_guild_channel_edit,
                    "ctx_thread_create_env": ctx_thread_create_env,
                    "ctx_thread_remove_env": ctx_thread_remove_env,
                    "ctx_thread_join_env": ctx_thread_join_env,
                    "ctx_thread_delete_env": ctx_thread_delete_env,
                    "ctx_thread_update_env": ctx_thread_update_env,
                    "ctx_thread_member_join_env": ctx_thread_member_join_env,
                    "ctx_thread_member_remove_env": ctx_thread_member_remove_env
                  
                }
                try:
                    result = await xfox.parse(codigo, del_empty_lines=True, **kwargs)
                except ValueError as e:
                    result = e

                botones, embeds = await utils.utils()
                view = discord.ui.View()
                if botones:
                    
                    # Crear un View para los botones
                    for boton in botones:
                        view.add_item(boton)  # Agregar los botones al View
                
                try:
                    if ctx_message_env:
                        await ctx_message_env.channel.send(result, 
                                                        view=view if view else None,  
                                                        embeds=embeds if embeds else [])

                    elif ctx_reaction_env:
                        channel = self.get_channel(ctx_reaction_env.channel_id, )
                        if channel:
                            await channel.send(result, 
                                            view=view if view else None,
                                            embeds=embeds if embeds else [])


                    elif ctx_reaction_remove_env:
                        channel = self.get_channel(ctx_reaction_remove_env.channel_id)
                        if channel:
                            await channel.send(result, 
                                            view=view,
                                            embeds=embeds if embeds else [])

                    elif ctx_interaction_env:
                        try:
                            mensaje_original = await ctx_interaction_env.channel.fetch_message(ctx_interaction_env.message.id)
                            view_original = discord.ui.View.from_message(mensaje_original)

       
                            for item in view_original.children:
                                view.add_item(item)
                        except Exception as e:
                                print(f"[DEBUG ONINTERACTION] Error al combinar views: {e}")
                        
           
                        await ctx_interaction_env.response.edit_message(
                                                            content=result,
                                                            view=view,
                                                            embeds=embeds
                                                                    )
   
                        

                    elif ctx_message_edit_env:
                        before, after = ctx_message_edit_env
                        await before.channel.send(content=result, 
                                              view=view, 
                                              embeds=embeds)

                    elif ctx_message_delete_env:
                        await ctx_message_delete_env.channel.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                    
                    elif ctx_bulk_message_delete_env:
                        
                        await ctx_bulk_message_delete_env.channel.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                        
                    elif ctx_typing_env:
                        
                        await ctx_typing_env[0].send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                    elif ctx_guild_channel_create:
                        
                        await ctx_guild_channel_create.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                    elif ctx_guild_channel_edit:
                        
                        await ctx_guild_channel_edit[0].send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                        
                    elif ctx_thread_create_env:
                        
                        await ctx_thread_create_env.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                        
                    elif ctx_thread_member_join_env:
                        
                        await ctx_thread_member_join_env.thread.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                        
                    elif ctx_thread_member_remove_env:
                        
                        await ctx_thread_member_remove_env.thread.send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                        
                    elif ctx_thread_update_env:
                        
                        await ctx_thread_update_env[1].send(content=result, 
                                                                    view=view,
                                                                    embeds=embeds)
                    
                    
                    await clear_data()
                except Exception as e:
                    print(f"[DEBUG ERRORCLIENT]: {e}")
    


    async def on_message(self, ctx_message_env):
        if ctx_message_env.author.bot:
            return
        
        utils.ContextAmisynth(ctx_message_env)

        await self.ejecutar_eventos("$onMessage", ctx_message_env)
        await self.process_commands(ctx_message_env)  # Permite que otros comandos de discord.py sigan funcionando




    async def on_ready(self):
        print(f"[DEBUG CLIENT] USER:{self.user}")
        utils.bot_id = self.user.id
        await self.ejecutar_eventos("$onReady")
        try:
            synced = await self.tree.sync()
            
        except Exception as e:
            print(f"Error al sincronizar slash commands: {e}")



    async def on_member_join(self, member: discord.Member):
        utils.ContextAmisynth(member)
        await self.ejecutar_eventos("$onJoinMember", ctx_join_member_env=member)
    
    async def on_member_remove(self, member: discord.Member):
        utils.ContextAmisynth(member)
        await self.ejecutar_eventos("$onLeaveMember", ctx_remove_member_env=member)


        
    async def on_raw_reaction_add(self, ctx_reaction_env: discord.RawReactionActionEvent):
        utils.ContextAmisynth(ctx_reaction_env)
        """Maneja cuando un usuario añade una reacción."""
        await self.ejecutar_eventos("$onReactionAdd", ctx_reaction_env=ctx_reaction_env)



    async def on_raw_reaction_remove(self, ctx_reaction_remove_env: discord.RawReactionActionEvent):
        """Maneja cuando un usuario remueve una reacción."""
        utils.ContextAmisynth(ctx_reaction_remove_env)
        await self.ejecutar_eventos("$onReactionRemove", ctx_reaction_remove_env=ctx_reaction_remove_env)



    async def on_interaction(self, ctx_interaction_env: discord.Interaction):
        """Maneja interacciones como botones y menús."""
        if ctx_interaction_env.user.bot:
            return
        utils.ContextAmisynth(ctx_interaction_env)
        await self.ejecutar_eventos("$onInteraction", ctx_interaction_env=ctx_interaction_env)



    async def on_message_edit(self, before, after):
        utils.ContextAmisynth(before)
        if before.author.bot:  # Evita que el bot procese sus propios mensajes
            return
        await self.ejecutar_eventos("$onMessageEdit", ctx_message_edit_env=(before, after))
        

    async def on_message_delete(self, message: discord.Message):
        if message.author.bot:
            return
        utils.ContextAmisynth(message)
        await self.ejecutar_eventos("$onMessageDelete", ctx_message_delete_env=message)



    async def on_bulk_message_delete(self, ctx_bulk_message_delete_env: discord.Message):
        utils.ContextAmisynth(ctx_bulk_message_delete_env)
        await self.ejecutar_eventos("$onMessagesPurged", ctx_bulk_message_delete_env=ctx_bulk_message_delete_env[0])

    async def on_typing(self, channel, user, when):
        utils.ContextAmisynth(channel)
        await self.ejecutar_eventos("$onMessageTyping", ctx_typing_env=(channel, user, when))

    async def on_guild_channel_create(self, channel: discord.TextChannel):
        utils.ContextAmisynth(channel)
        await self.ejecutar_eventos("$onChannelCreate", ctx_guild_channel_delete=channel)


    async def on_guild_channel_delete(self, channel: discord.TextChannel):
        utils.ContextAmisynth(channel)
        await self.ejecutar_eventos("$onChannelDelete", ctx_guild_channel_delete=channel)

    async def on_guild_channel_update(self, before, after):
        utils.ContextAmisynth(before)
        await self.ejecutar_eventos("$onChannelEdit", ctx_guild_channel_edit=(before, after))


    async def on_thread_create(self, thread):
        utils.ContextAmisynth(thread)
        await self.ejecutar_eventos("$onThreadCreate", ctx_thread_create_env=thread)

    
    async def on_thread_remove(self, thread):
        utils.ContextAmisynth(thread)
        await self.ejecutar_eventos("$onThreadRemove", ctx_thread_remove_env=thread)

    
    async def on_thread_join(self, thread):
        utils.ContextAmisynth(thread)
        await self.ejecutar_eventos("$onThreadJoin", ctx_thread_join_env=thread)


    async def on_thread_delete(self, thread):
        utils.ContextAmisynth(thread)
        await self.ejecutar_eventos("$onThreadDelete", ctx_thread_delete_env=thread)


    async def on_thread_update(self, before, after):
        utils.ContextAmisynth(before)
        await self.ejecutar_eventos("$onThreadUpdate", ctx_thread_update_env=(before, after))

    
    async def on_thread_member_join(self, thread_member):
        utils.ContextAmisynth(thread_member)
        await self.ejecutar_eventos("$onThreadJoinMember", ctx_thread_member_join_env=thread_member)

    async def on_thread_member_remove(self, thread_member):
        utils.ContextAmisynth(thread_member)
        await self.ejecutar_eventos("$onThreadRemoveMember", ctx_thread_member_remove_env=thread_member)



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def categoryIDs(separator:str, guild_id=None, *args, **kwargs):
    contexto = utils.ContextAmisynth()

    # Si no se proporciona guild_id, usamos el del contexto
    if guild_id is None:
        guild_id = contexto.guild_id

    # Obtenemos los nombres de los canales
    nombres = contexto.get_categorys_ids(guild_id=int(guild_id))
    # Validamos si hay canales
    if not nombres:
        return ""

    # Unimos los nombres con el separador
    return separator.join(nombre for nombre in nombres)



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArrayPop(*claves, **kwargs):
    if not claves:
        return ""  # Si no se pasan claves, devuelve vacío
    
    data = json_storage

    try:
        for clave in claves:
            if clave not in data:
                return ""  # Si la clave no existe, devuelve vacío
            data = data[clave]  # Avanza en la estructura del JSON
        
        if isinstance(data, list) and data:
            return str(data.pop())  # Elimina y devuelve el último elemento
        return ""  # Si no es un array o está vacío, devuelve vacío
    except Exception as e:
        raise ValueError(f"Error al hacer pop en el array: {e}")



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def userRoles(separator: str = None, option: str = "names", user_id: int = None, *args, **kwargs):
    contexto = utils.ContextAmisynth()

    # Asigna el user_id si no se proporciona
    resolved_user_id = user_id if user_id is not None else contexto.author_id

    # Obtiene el miembro del guild
    member = await contexto.obj_guild.fetch_member(resolved_user_id)

    # Establece un separador por defecto si no se proporciona
    if separator is None:
        separator = "\n"

    # Filtra los roles, excluyendo @everyone
    roles = [rol for rol in member.roles if rol.name != "@everyone"]

    # Devuelve los datos según el tipo de opción
    if option == "names":
        return separator.join(rol.name for rol in roles)
    elif option == "ids":
        return separator.join(str(rol.id) for rol in roles)
    
    else:
        return f"Opción inválida: '{option}'. Usa 'names' o 'ids'."



import xfox
import discord
from Amisynth.utils import ContextAmisynth
@xfox.addfunc(xfox.funcs)
async def channelSystem(*args, **kwargs):
    ctx = ContextAmisynth()
    n = ctx.obj_guild.system_channel.id
    return n




import re
import xfox

@xfox.addfunc(xfox.funcs)
async def checkCondition(condition: str, *args, **kwargs):
    operadores_validos = ["!=", "==", ">=", "<=", ">", "<"]
    
    # Expresión regular para capturar la condición
    patron = r'^(.*?)(!=|==|>=|<=|>|<)(.*?)$'
    coincidencia = re.match(patron, condition)
    
    if not coincidencia:
        return "False"  # Retorna False si la condición no es válida
    
    izquierda, operador, derecha = coincidencia.groups()
    izquierda, derecha = izquierda.strip(), derecha.strip()
    
    # Si alguno de los valores es vacío, se deja como cadena vacía
    izquierda = izquierda if izquierda else ""
    derecha = derecha if derecha else ""
    
    # Caso especial: Si un lado está vacío y el otro no, evitar evaluar "e==" como True
    if (izquierda == "" and derecha != "") or (derecha == "" and izquierda != ""):
        return "False" if operador == "==" else "True"
    
    try:
        # Intentar convertir a número si es posible, si no, dejar como string
        izquierda = float(izquierda) if izquierda.replace(".", "", 1).isdigit() else izquierda
        derecha = float(derecha) if derecha.replace(".", "", 1).isdigit() else derecha
        
        # Evaluar la condición
        resultado = eval(f'"{izquierda}" {operador} "{derecha}"') if isinstance(izquierda, str) or isinstance(derecha, str) else eval(f'{izquierda} {operador} {derecha}')
        
        return "True" if resultado else "False"
    except Exception:
        return "False"



import re
import xfox
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def argsCheck(check, error_msg, *args, **kwargs):
    """
    Verifica si el número de argumentos proporcionados cumple con la condición especificada.
    Si no se cumple la condición, se lanza un ValueError con un mensaje de error personalizado.

    :param check: Condición para la validación como '>n', '<n', o 'n' (donde n es un número).
    :param error_msg: El mensaje de error que se lanza si la condición no se cumple.
    :param args: Los argumentos proporcionados por el usuario en la lista.
    """
    
    # Obtener los argumentos proporcionados por el usuario

    context = utils.ContextAmisynth()
    user_arguments = context.arguments

    # Extraer la operación y el número del check (ejemplo: '>3', '<5', '3')
    operator = check[0]  # El operador (> o < o =)
    number = int(check[1:])  # El número después del operador

    # Verificar la condición y lanzar el error si no se cumple
    if operator == ">":
        if len(user_arguments) <= number:
            raise ValueError(error_msg)
    elif operator == "<":
        if len(user_arguments) >= number:
            raise ValueError(error_msg)
    elif operator == "=":
        if len(user_arguments) != number:
            raise ValueError(error_msg)
    else:
        raise ValueError(f"Condición no válida en $argsCheck[{check};..]. Usa '>n', '<n' o 'n'.")
    
    return ""  # Retornar vacío si no hubo error



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonArrayCount(*claves, **kwargs):
    if not claves:
        raise ValueError("Error: Debes proporcionar al menos una clave en $jsonArrayCount[]")
    
    data = json_storage
    
    try:
        for clave in claves:
            if clave not in data:
                return "-1"
            data = data[clave]  # Avanza en la estructura del JSON
        
        return str(len(data)) if isinstance(data, list) else "-1"
    except Exception as e:
        raise ValueError(f"Error al contar elementos en el array: {e}")



import xfox
from Amisynth.utils import options_slash


@xfox.addfunc(xfox.funcs)
async def addSlashChoiceOption(nombre_opcion: str, name_choice: str, value_choice: str, *args, **kwargs):
    """
    Agrega un choice a una opción existente.

    Uso:
        $add_choice[mensaje;josue;Josuehee]
    """
    if nombre_opcion is None:
        raise ValueError(":x: Error en `$addSlashChoiceOption[?;]` primer agrumento vacio")
    elif name_choice is None:
        raise ValueError(":x: Error en `$addSlashChoiceOption[.;.?]` segundo agrumento vacio")
    elif value_choice is None:
        raise ValueError(":x: Error en `$addSlashChoiceOption[.;..;?]` tercer agrumento vacio")
    for opcion in options_slash:
        if opcion["name_option"] == nombre_opcion:
            opcion.setdefault("choices", []).append({
                "name_choice": name_choice,
                "value_choice": value_choice
            })
            print("[DEBUG ADDCHOICEOPTION] Choice Agregado correctamente.")
            return ""
    print("[SLASH] No se encontró una opción con nombre '{nombre_opcion}' para agregar choice.")
    raise ValueError(f":x: No se encontró una opción con nombre '{nombre_opcion}' para agregar choice.")


from setuptools import setup, find_packages

setup(
    name='Amisynth',  
    version='0.3.8',    
    packages=find_packages(include=["Amisynth", "Amisynth.*"]),    
    py_modules=["Amisynth"],  # Agrega esto si solo tienes archivos sueltos sin una estructura de paquete
    install_requires=["discord.py", "asyncio", "xfox", "youtube_search", "aiohttp"],  
    description='Crea tu bot de discord sin saber programar!',
    long_description=open('README.md', encoding='utf-8').read(),  
    long_description_content_type='text/markdown',
    author='Amisinth',
    author_email='amisynth@gmail.com',
    url='https://github.com/Amisynth/Amisynth',  
    classifiers=[  
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',  
)



import xfox
import discord
import Amisynth.utils as utils
from Amisynth.utils import utils as utils_func
from Amisynth.utils import buttons as but, embeds as emb

@xfox.addfunc(xfox.funcs, name="eval")
async def eval_command(code=None, *args, **kwargs):
    # ✅ Verificación para evitar que 'code' sea None
    if code is None:
        return ""

    context = utils.ContextAmisynth()
    channel = context.obj_channel

    try:
        code = await xfox.parse(code, del_empty_lines=True)

    except ValueError as e:
        return f"{e}"


    texto = code

    botones, embeds = await utils_func()

    view = discord.ui.View()
    if botones:
        for boton in botones:
            view.add_item(boton)
    print(f"[DEBUG EVAL - CHANNEL] Canal ejeciutado: {channel.name}")
    await channel.send(
        content=texto if texto else "",
        view=view,
        embeds=embeds if embeds else []
    )
    from Amisynth.utils import clear_data
    await clear_data()

    return ""




import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def authorID(nombre: str = None, *args, **kwargs):
    context = utils.ContextAmisynth()
    if nombre is None:
        author_id  = context.author_id
    else:
        author_id = context.get_user_id_by_username(nombre)

    return author_id



import xfox
import Amisynth.utils as utils
import re

@xfox.addfunc(xfox.funcs)
async def findChannel(query: str, *args, **kwargs):
    context = utils.ContextAmisynth()
    guild = context.obj_guild

    if guild is None:
        print("[DEBUG FINDCHANNEL] No se puede buscar fuera de un servidor.")
        raise ValueError(":x: No se puede buscar fuera de un servidor.")

    query = query.strip()

    # 🟦 Si es mención de canal: <#123456789012345678>
    mention_match = re.match(r"<#(\d+)>", query)
    if mention_match:
        channel_id = int(mention_match.group(1))
        channel = guild.get_channel(channel_id)
        if channel:
            print(f"[DEBUG FINDCHANNEL] Encontrado por mención: {channel}")
            return str(channel.id)

    # 🟩 Si es ID
    if query.isdigit():
        channel = guild.get_channel(int(query))
        if channel:
            print(f"[DEBUG FINDCHANNEL] Encontrado por ID: {channel}")
            return str(channel.id)

    # 🟨 Si es nombre de canal
    query_lower = query.lower()
    for channel in guild.channels:
        if channel.name.lower() == query_lower:
            print(f"[DEBUG FINDCHANNEL] Encontrado por nombre: {channel}")
            return str(channel.id)

    print("[DEBUG FINDCHANNEL] Canal no encontrado")
    return ""



import xfox
import re

@xfox.addfunc(xfox.funcs)
async def findNumbers(text: str, *args, **kwargs):
    # Busca todos los números en el texto
    numbers = re.findall(r"\d+", text)
    
    return ', '.join(numbers) if numbers else ""



import xfox
from Amisynth.utils import json_storage

@xfox.addfunc(xfox.funcs)
async def jsonJoinArray(*claves_y_separador, **kwargs):
    if len(claves_y_separador) < 2:
        return ""  # Si no hay suficientes parámetros, devuelve vacío
    
    *claves, separador = claves_y_separador  # Extrae claves y separador
    data = json_storage

    try:
        for clave in claves:
            if clave not in data or not isinstance(data[clave], (dict, list)):
                return ""  # Si alguna clave no existe o no es válida, devuelve vacío
            data = data[clave]  # Avanza en la estructura del JSON

        if not isinstance(data, list):
            return ""  # Si el resultado no es una lista, devuelve vacío

        return separador.join(map(str, data))  # Une los elementos con el separador
    except Exception as e:
        raise ValueError(f"Error en $jsonJoinArray: {e}")



import xfox
from Amisynth.utils import embeds
import re  # Asegúrate de que 'embeds' sea la lista global que deseas modificar
URL_REGEX = re.compile(r"^(https?|ftp):\/\/[^\s/$.?#].[^\s]*$", re.IGNORECASE)
@xfox.addfunc(xfox.funcs)
async def thumbnail(url: str, indice: int = 1, *args, **kwargs):
    """
    Guarda un thumbnail en la lista de embeds, con una URL de imagen específica y un índice opcional.
    Si se especifica el índice, se inserta o actualiza en esa posición. Si no, se agrega en la posición 1.
    """

    if url is None:
        print("[DEBUG THUMBNAIL] Esta vacia.")
        raise ValueError(f":x: URL Vacia en $thumbnail[]")
    
    elif not URL_REGEX.match(url):
        print("[DEBUG THUMBNAIL] URL inválida o None:", url)
        raise ValueError(f":x: URL no válida en $thumbnail[{url}]")
    
    embed = {
        "thumbnail_icon": url,  # URL de la imagen como thumbnail
        "index": indice    # Añadir el índice para identificar la posición
    }

    # Buscar si ya existe un embed con ese índice y actualizar solo el thumbnail
    found = False
    for i, item in enumerate(embeds):
        if item.get("index") == indice:
            # Mantener los otros atributos del embed y solo actualizar el thumbnail
            embeds[i]["thumbnail_icon"] = url
            found = True
            break
    if not found:
        # Si no se encontró, agregar uno nuevo
        embeds.append(embed)

    return ""



import math
import xfox

@xfox.addfunc(xfox.funcs, name="sqrt")
async def square_root(number: str, *args, **kwargs):
    try:
        num = float(number)
        if num < 0:
            raise ValueError(":x: No se puede calcular la raíz cuadrada de un número negativo.")
        return str(math.sqrt(num))
    except ValueError:
        return f":x: Invalid number value: '{number}' en `$sqrt[]`"



import xfox

import time

@xfox.addfunc(xfox.funcs)
async def getTimestamp(opcion=None, *args, **kwargs):
    # Obtener el tiempo actual
    current_time = time.time()  # Esto devuelve el tiempo en segundos (float)

    if opcion: 
        if opcion.lower() == "s":
            return str(int(current_time))  # Devuelve el tiempo en segundos (s)
    
        elif opcion.lower() == "ms":
            return str(int(current_time * 1000))  # Convierte a milisegundos (ms)
    
        elif opcion.lower() == "ns":
             return str(int(current_time * 1e9))  # Convierte a nanosegundos (ns)
    
        else:
            raise ValueError(":x: Opción no válida en `$getTimestamp[]`. Usa `s`, `ms`, o `ns`.")
        
    elif opcion is None:
        return str(int(current_time))



import xfox

@xfox.addfunc(xfox.funcs, name="power")
async def power_of_number(base: str, exponent: str, **kwargs):
    try:
        base_num = float(base)
        exponent_num = float(exponent)
        return str(base_num ** exponent_num)
    except ValueError:
        return f":x: Invalid number value en `$power[]`"



import xfox
import discord
import Amisynth.utils as utils

@xfox.addfunc(xfox.funcs)
async def removeComponent(custom_id: str, message_id=None, *args, **kwargs):
    """Elimina un componente (botón o select menu) del mensaje por su custom_id.
       Si el custom_id es '-1', elimina todos los componentes."""
    
    if message_id is None:
        raise ValueError("Debes proporcionar un ID de mensaje para $removeComponent.")

    context = utils.ContextAmisynth()
    message = await context.get_message_from_id(int(message_id))

    # Obtener la vista actual
    view = discord.ui.View() if not message.components else discord.ui.View.from_message(message)

    # Si el custom_id es "-1", limpiar todos los componentes
    if custom_id == "-1":
        await message.edit(view=discord.ui.View())  # Vista vacía = sin componentes
        return ""

    # Si no, eliminar solo el componente con el custom_id dado
    new_children = [
        item for item in view.children
        if not (
            hasattr(item, "custom_id") and item.custom_id == custom_id
        )
    ]

    # Crear nueva vista con los componentes restantes
    new_view = discord.ui.View()
    for item in new_children:
        new_view.add_item(item)

    await message.edit(view=new_view)
    return ""



import xfox
import discord
import Amisynth.utils as utils
@xfox.addfunc(xfox.funcs)
async def setUserVar(nombre=None, valor=None, user_id=None, guild_id=None,  *args, **kwargs):
    context = utils.ContextAmisynth()
    if nombre is None:
        raise ValueError(":x: Error el nombre esta vacio de la `$setUserVar[?;..]`")
    
    if valor is None:
         raise ValueError(":x: Error el valor esta vacio de la funcion`$setUserVar[..;?]`")
    
    if user_id is None:
        user_id = context.author_id

    if guild_id is None:
        guild_id = context.guild_id

    var = utils.VariableManager()

    value = var.set_value("user", key=nombre, value=valor, user_id=user_id, guild_id=guild_id)
    return ""